;*****************************************************************************
;|									     |
;|  PROGARM	: H11A17HM.ASM						     |
;|									     |
;|  FUNCTIONS	: Hub/EmbeddedFunction Control Program			     |
;|									     |
;|  COMPILER	: ASM51 Macro Assebbler V1.2 ( SW I2C version)		     |
;|									     |
;|  USED IN	: 80C51 						     |
;|									     |
;|  AUTHORS	: Eric Lu						     |
;|									     |
;|  WRITTEN AT	: 15-11-96						     |
;|									     |
;|  VERSION	: V 1.7 						     |
;|									     |
;|  UPDATES	: 05-14-98						     |
;|									     |
;|The sample code is provided on an "as is" basis, and Philips Semiconductors |
;|expressly disclaims any or all warranties expressed or implied, including   |
;|without limitation warranties of fitness for a particular purpose. In no    |
;|event shall Philips Semiconductors be liable for any direct, indirect,      |
;|incidental consequential damages of any kind whatsover with respect to the  |
;|use of the code.										|
;|													|
;|Philips Semiconductors reserves the right, in its sole discretion and		|
;|without any obligation, to make improvements to, or correct any error in any|
;|portion of the code.										|
;|									     |
;*****************************************************************************
; Code History
; H11A10HM.asm, First released version for Hub with embedded function using H11A
; H11A11HM.asm, Added a Hot Key to pop-up applet
; H11A12HM.asm, Added 2 more keys for Volume control and store DAC data in uC
; H11A13HM.asm, Added a security key, compliance to RC7
; H11A14HM.asm, Fixed the Set Address command and individual OV problem
; H11A15HM.asm: 1. Changed to after Set_Address command, uC wait for interrupt
;		   pin active( low) then change Hub address( For Hub), for embedded
;		   need to check remaining interrupt task and remember it.
;		2. Sent Zero-length packet to host, when all data were all sent
;		   for host but IN token is still coming.
;		3. Added auto-detection between H11 and H12.
; H11A16HM.asm, Supporting single Hot Key on small VGA board.
; H11A17HM.asm, Changed to no remote wakeup feature in embedded function
;*****************************************************************************
$TITLE(H11 USB Hub)
$PAGEWIDTH(132)
$DATE(12/1/1995)
$DEBUG
$OBJECT
;$NOPAGING
$MOD51
;----------------
FLAG		DATA	20H		;I2C SOFTWARE STATUS FLAGS.
I2C_FAIL	BIT	FLAG.0		;I2C FAIL FLAG.
INIT_FLAG	BIT	FLAG.1		;VALIDATE BUFFER FLAG
SEL_EPT_FLAG	BIT	FLAG.2		;SEL HUB OR EMBEDDED FLAG
HOT_KEY 	BIT	FLAG.3
H12_FLAG	BIT	FLAG.4
;		BIT	FLAG.5
;		BIT	FLAG.6
;----------------
HUB_STATUS	DATA	21H		;FLAG AND STATUS
SELF_POWER	BIT	HUB_STATUS.0	;HUB SELF POWERED FLAG
REMOTE_WAKE	BIT	HUB_STATUS.1	;HUB REMOTE WAKEUP FLAG
EPT0_OUT_STALL	BIT	HUB_STATUS.2	;HUB EPT0 OUT STALL FLAG
EPT0_IN_STALL	BIT	HUB_STATUS.3	;HUB EPT0 IN STALL FLAG
EPT1_IN_STALL	BIT	HUB_STATUS.4	;HUB EPT1 IN STALL FLAG
HUB_CONFIG	BIT	HUB_STATUS.5	;HUB CONFIGURED
VALIDATE_FLAG	BIT	HUB_STATUS.6	;WRITE BUFFER FLAG
PWR_SRC_FLAG	BIT	HUB_STATUS.7	;

;----------------
EMB_STATUS	DATA	22H		;FLAG AND STATUS
EMB_SELF_POWER	    BIT     EMB_STATUS.0   ;EMB FUNCTION SELF POWERED FLAG
EMB_REMOTE_WAKE     BIT     EMB_STATUS.1   ;EMB FUNCTION REMOTE WAKEUP FLAG
EMB_EPT0_OUT_STALL  BIT     EMB_STATUS.2   ;EMB FUNCTION EPT0 OUT STALL FLAG
EMB_EPT0_IN_STALL   BIT     EMB_STATUS.3   ;EMB FUNCTION EPT0 IN STALL FLAG
EMB_EPT1_IN_STALL   BIT     EMB_STATUS.4   ;EMB FUNCTION EPT1 IN STALL FLAG
EMB_CONFIG	    BIT     EMB_STATUS.5   ;EMB CONFIGURED
;----		    BIT     EMB_STATUS.6   ;
;----		    BIT     EMB_STATUS.7   ;

;----------------
EMBED_STATUS0	DATA	23H	;CONTAINS THE STATUS OF PORT1
;----------------
E_PORT_CONN	BIT	EMBED_STATUS0.0
E_PORT_ENABLE	BIT	EMBED_STATUS0.1
E_PORT_SUSPEND	BIT	EMBED_STATUS0.2
E_PORT_OVERCUR	BIT	EMBED_STATUS0.3
E_PORT_RESET	BIT	EMBED_STATUS0.4
E_PORT_POWER	BIT	EMBED_STATUS0.5
E_LOW_SPEED	BIT	EMBED_STATUS0.6
;---------------------------------
EMBED_STATUS1	DATA	24H
;--------------
E_C_PORT_CONN	  BIT	EMBED_STATUS1.0
E_C_PORT_ENABLE   BIT	EMBED_STATUS1.1
E_C_PORT_SUSPEND  BIT	EMBED_STATUS1.2
E_C_PORT_OVERCUR  BIT	EMBED_STATUS1.3
E_C_PORT_RESET	  BIT	EMBED_STATUS1.4
;---------
KEY_STATUS	DATA	25H
;----------------
DSEG	AT	30H
;-----------------
RW_COUNTER	DATA	30H
HUB_ADDR	DATA	31H	;address of the hub
LAST_COM	DATA	32H	;CONTAINS THE LAST RECEIVED REQUEST
BYTE_TO_SEND	DATA	33H
MSG_DPH 	DATA	34H
MSG_DPL 	DATA	35H
;--------------------------
EMB_ADDR	DATA	36H	;ADDRESS OF THE EMBEDDED FUNCTION
EMB_LAST_COM	DATA	37H	;CONTAINS THE LAST RECEIVED REQUEST
EMB_BYTE_TO_SEND  DATA	38H	;
EMB_MSG_DPH	DATA	39H
EMB_MSG_DPL	DATA	3AH
DAC_CHANNEL	DATA	3BH
;---------------------------------
COUNTER 	DATA	3CH		;COUNTER
INT_BYTE_TO_SEND	DATA	3DH
I2C_COUNTER	DATA	3EH
;-------------------------------------------------
;  S/W I2C CONTROL REGISTERS
;-------------------------------------------------
BITCNT		DATA	40H		;I2C BIT COUNTER.
BYTE_CNT	DATA	41H		;BYTES TO BE PROCESSED BY I2C TX/RX
CNT_BYTE	DATA	42H		;NUMBER OF BYTES TO BE TRANSMITTED
SLV_ADR 	DATA	43H		;I2C ADDRESS OF ACTIVE SLAVE.
I2C_BUFFER	DATA	44H		;I2C TRANSMIT/RECEIVE DATA BUFFER

;----------------------------		;64H-6FH 10 BYTES
BRIT_DAC	DATA	50H		;DA CHANNEL A
CONT_DAC	DATA	51H		;DA CHANNEL B
HBAL_DAC	DATA	52H
RG_DAC		DATA	53H		;DA CHANNEL 7
GG_DAC		DATA	54H		;DA CHANNEL 8
BG_DAC		DATA	55H		;DA CHANNEL 9
HP_DAC		DATA	56H		;DA CHANNEL 0
HS_DAC		DATA	57H		;DA CHANNEL 1
HPIN_DAC	DATA	58H
VC_DAC		DATA	59H		;DA CHANNEL 2
VS_DAC		DATA	5AH		;DA CHANNEL 3
VPIN_DAC	DATA	5BH
PARA_DAC	DATA	5CH
TZ_DAC		DATA	5DH		;DA CHANNEL 5
ROT_DAC 	DATA	5EH		;DA CHANNEL 6
;-----------------
STACK_BUF	DATA	5FH
;*******************************************
; H11 commands
;
;
; Initialization Commands
;
SET_HUB_ADDRESS_EN	EQU	0D0h
SET_EMB_ADDRESS_EN	EQU	0D1h
SET_EPOINT_EN		EQU	0D8h

;
; Data Flow Commands
;
SEL_EPOINT_0_OUT	EQU	000h
SEL_EPOINT_0_IN 	EQU	001h
SEL_EPT_EMBF_OUT	EQU	002h
SEL_EPT_EMBF_IN 	EQU	003h
SEL_EPT_EMBF_INT	EQU	004h
;---
READ_LAST_EPT0_OUT_ST	EQU	040h
READ_LAST_EPT0_IN_ST	EQU	041h
READ_LAST_EMBF_OUT_ST	EQU	042h
READ_LAST_EMBF_IN_ST	EQU	043H
READ_LAST_EMBF_INT_ST	EQU	044H
;---
READ_EPT0_OUT_ST	EQU	080h
READ_EPT0_IN_ST 	EQU	081h
READ_EMBF_OUT_ST	EQU	082h
READ_EMBF_IN_ST 	EQU	083H
READ_EMBF_INT_ST	EQU	084H
;---
SET_EPT0_OUT_ST 	EQU	040h
SET_EPT0_IN_ST		EQU	041h
SET_EMBF_OUT_ST 	EQU	042h
SET_EMBF_IN_ST		EQU	043H
SET_EMBF_INT_ST 	EQU	044H
;---
READ_BUFFER		EQU	0F0h
WRITE_BUFFER		EQU	0F0h
ACK_SETUP		EQU	0F1h
CLEAR_BUF		EQU	0F2h
READ_INT		EQU	0F4h
VALIDATE_BUF		EQU	0FAh
;---
;-----------------------------------
; Hub Commands
;
CLEAR_PORT_F		EQU	0E0h
CLEAR_PORT_2		EQU	0E0h
CLEAR_PORT_3		EQU	0E1h
CLEAR_PORT_4		EQU	0E2h
CLEAR_PORT_5		EQU	0E3h
;---
GET_PORT_STATUS_F	EQU	0E0h
GET_PORT_STATUS_2	EQU	0E0h
GET_PORT_STATUS_3	EQU	0E1h
GET_PORT_STATUS_4	EQU	0E2h
GET_PORT_STATUS_5	EQU	0E3h
;---
SET_STATUS_CHG_BITS	EQU	0F7h

SET_PORT_F		EQU	0E8h
SET_PORT_2		EQU	0E8h
SET_PORT_3		EQU	0E9h
SET_PORT_4		EQU	0EAh
SET_PORT_5		EQU	0EBh

;
; General Commands
;
SEND_RESUME		EQU	0F6h
READ_CURR_FRAME_NUM	EQU	0F5h
SET_MODE		EQU	0F3H

;----------------------------------------
OFFSET_bmReqTyp 	EQU	02h
OFFSET_bmReq		EQU	03h
OFFSET_wValue		EQU	04h
OFFSET_wIndex		EQU	06h
OFFSET_wLenght		EQU	08h
;---------------------------------------
REQ_SET_ADDRESS 	EQU	05H
REQ_SET_REPORT		EQU	01H
REQ_GET_REPORT		EQU	02H
REQ_SET_FEATURE 	EQU	03H
REQ_GET_FEATURE 	EQU	04H
;---------------------------------------
;**************************************
; Port Feature Code
;**************************************
F_PORT_ENABLE	EQU		00h
F_PORT_SUSPEND	EQU		01h
FC_PORT_RESET	EQU		02h
F_PORT_POWER	EQU		03h
C_PORT_CONN	EQU		04h
C_PORT_ENABLE	EQU		05h
C_PORT_SUSPEND	EQU		06h
C_PORT_OVERCUR	EQU		07h

;--------------------------------------
report_counter1   equ	  08
;--------------------------------------
ENS1_NOTSTA_STO_NOTSI_NOTAA_CR0 	     EQU	0D1H
ENS1_NOTSTA_STO_NOTSI_AA_CR0		     EQU	0D5H
ENS1_NOTSTA_NOTSTO_NOTSI_AA_CR0 	     EQU	0C5H
ENS1_NOTSTA_NOTSTO_NOTSI_NOTAA_CR0	     EQU	0C1H
ENS1_STA_NOTSTO_NOTSI_AA_CR0		     EQU	0E5H
;----------------------------------------
REPORT_LENGTH			EQU	0FFH
;----------------------------------------
; IO PORT Definition
;---------------------------------------
SCLPIN		BIT	P0.0		;S/W I2C CLOCK PIN
SDAPIN		BIT	P0.1		;S/W I2C DATA PIN
H11_INT_PORT	BIT	P1.5		;H11 INTERRUPT PIN
SUSPEND_PORT	BIT	P1.2		;H11 SUSPEND PIN
POWER_INPUT_PIN BIT	P1.0		;BUS/SELF POWER INDICATION PIN
HWSCL		BIT	P1.6
HWSDA		BIT	P1.7
HOT_KEY_1	BIT	P1.1
VOL_UP_KEY	BIT	P1.3
VOL_DN_KEY	BIT	P1.4
;-----------------------------
; INTERRUPT VECTOR TABLE
;------------------------------
CSEG	AT	0000H
	ORG	0000H
	AJMP	START			;PROGRAM START
	ORG	0003H
	AJMP	INT0_SERVER		;INT0 SERVICE ROUTINE
	ORG	000BH
	AJMP	T0_SERVER		;TIMER0 SERVICE ROUTINE
	ORG	0013H
	AJMP	INT1_SERVER		;INT1 SERVICE ROUTINE
	ORG	001BH
	AJMP	T1_SERVER		;TIMER1 SERVICE ROUTINE
	ORG	0023H
	AJMP	UART_SERVER		;UART SERVICE ROUTINE
	ORG	002BH
	AJMP	I2C_SERVER		;I2C SERVICE ROUTINE

;******************************
INT0_SERVER:				;INT0 SERVICE ROUTINE
T0_SERVER:				;TIMER0 SERVICE ROUTINE
INT1_SERVER:				;INT1 SERVICE ROUTINE
T1_SERVER:				;TIMER1 SERVICE ROUTINE
UART_SERVER:				;UART SERVICE ROUTINE
I2C_SERVER:				;I2C SERVICE ROUTINE
	RETI

;***************************************
; PROGRAM START
;***************************************
	ORG	0030H
START:
	MOV	IE,#00H 		;ALL INTERRUPT DISABLE
	MOV	IP,#00H 		;INT PRIORITY DISABLE
	MOV	SP,#STACK_BUF		;SET STACK POINTER

;-----------------------------
; INITIAL I/O PORT
;-----------------------------
	MOV	P0,#11111111B		;
	MOV	P1,#11111111B
	MOV	P2,#11111111B
	MOV	P3,#11111111B

;-----------------------------
;CLEAR DATA MEMORY
;-----------------------------
	MOV	R0,#05H
CLR_DATA:
	MOV	@R0,#0
	INC	R0
	CJNE	R0,#080H,CLR_DATA
;-----------------
	MOV	I2C_BUFFER,#SET_MODE
	MOV	I2C_BUFFER+1,#10000110B 	;SWITCH TO H11 COMPATIBILE MODE
	MOV	I2C_BUFFER+2,#00000011B 	;SWITCH TO 12 MHZ OUTPUT
	ACALL	WRITE_TWO_BYTE			;REMOTE WAKEUP DISABLE
;------------------
	LCALL	DELAY_100MS
;------------------
;;	MOV	I2C_BUFFER,#0E0H		;CLEAR POWER
;;	MOV	I2C_BUFFER+1,#F_PORT_POWER
;;	ACALL	WRITE_ONE_BYTE
;------------------
	MOV	I2C_BUFFER,#0E2H
	ACALL	READ_ONE_BYTE
	MOV	A,I2C_BUFFER
	JNB	ACC.6,H11A_CONNT
;----
	MOV	I2C_BUFFER,#0E3H
	ACALL	READ_ONE_BYTE
	MOV	A,I2C_BUFFER
	JNB	ACC.6,H11A_CONNT
;---
	SETB	H12_FLAG		;H12 CONNECT
	AJMP	H11A_CONNT1
;---
H11A_CONNT:
	CLR	H12_FLAG
;------------------
H11A_CONNT1:
;------------------
	ACALL	INITIAL
;----
	MOV	COUNTER,#70H
;------
	MOV	BRIT_DAC,#127
	MOV	CONT_DAC,#127
	MOV	RG_DAC,#127
	MOV	GG_DAC,#127
	MOV	BG_DAC,#127
	MOV	HP_DAC,#51
	MOV	HS_DAC,#84
	MOV	HPIN_DAC,#127
	MOV	VS_DAC,#59
	MOV	VS_DAC,#55
	MOV	TZ_DAC,#58
	MOV	ROT_DAC,#62

;------------------
MAIN:
	LCALL	HOT_KEY_DT
	ACALL	USB_SUBROUTINE
	AJMP	MAIN


;--------------------
; INITIAL SETTING
;--------------------
INITIAL:
	MOV	HUB_STATUS,#01H        ; SELF POWER HUB
	MOV	EMB_STATUS,#01H
	MOV	HUB_ADDR,#00H
	MOV	EMB_ADDR,#00H
	MOV	EMBED_STATUS0,#00H
	MOV	EMBED_STATUS1,#00H
;----
	CLR	POWER_INPUT_PIN
	JB	POWER_INPUT_PIN,SELF_PWR_IN ;HUB IS SELF POWER HUB
	SETB	POWER_INPUT_PIN
	MOV	HUB_STATUS,#00H        ; BUS POWER HUB, REMOTE WAKEUP DISABLE
SELF_PWR_IN:
	SETB	POWER_INPUT_PIN
;----
	MOV	I2C_BUFFER,#SET_MODE		;RECONNECT SOFTCONNECT
	MOV	I2C_BUFFER+1,#10110100B 	;INITIALIZE H11A
	ACALL	WRITE_ONE_BYTE
	RET


;=========================================================================

USB_SUBROUTINE:
	JB	H11_INT_PORT,H11_RET		;IF H11 INT PORT HI THEN RETURN
;------
	MOV	I2C_BUFFER,#READ_INT		;READ INTERRUPT REGISTER
	ACALL	READ_TWO_BYTE			;READ DATA FROM H11
;------
	MOV	A,I2C_BUFFER
	JB	ACC.0,HUB_ED0_OUTPUT_P
	JB	ACC.1,HUB_ED0_INPUT
	JB	ACC.2,EMB_ED0_OUTPUT_P
	JB	ACC.3,EMB_ED0_INPUT
	JB	ACC.4,EMB_ED1_INPUT_P
;-----
	MOV	A,I2C_BUFFER+1
	JB	ACC.6,RESTART_H11
	RET
;--------------------
RESTART_H11:
;----
	MOV	I2C_BUFFER,#SET_HUB_ADDRESS_EN	;SET HUB ADDRESS : 0
	MOV	I2C_BUFFER+1,#080H		;ENABLE HUB
	ACALL	WRITE_ONE_BYTE
;----
	MOV	I2C_BUFFER,#SET_EPOINT_EN	;ENABLE HUB INTERRUPT ENDPOINT
	MOV	I2C_BUFFER+1,#01H
	ACALL	WRITE_ONE_BYTE
;-----
	MOV	I2C_BUFFER,#SET_EMB_ADDRESS_EN	;DISABLE EMBED ADDRESS
	MOV	I2C_BUFFER+1,#00
	ACALL	WRITE_ONE_BYTE
;----
	ACALL	INITIAL
;----
H11_RET:
DATA_ERROR:
	RET
;========================
EMB_ED0_OUTPUT_P:
	AJMP	EMB_ED0_OUTPUT
HUB_ED0_OUTPUT_P:
	AJMP	HUB_ED0_OUTPUT
EMB_ED1_INPUT_P:
	LJMP	EMB_ED1_INPUT


;********************************************************************
HUB_ED0_INPUT:
;---
	CLR	SEL_EPT_FLAG			    ;SELECT HUB BUFFER
;---
	MOV	I2C_BUFFER,#READ_LAST_EPT0_IN_ST     ;READ ENDPOINT STATUS FROM H11
	ACALL	READ_ONE_BYTE
;------
	MOV	I2C_BUFFER,#SEL_EPOINT_0_IN
	ACALL	READ_ONE_BYTE
	MOV	A,I2C_BUFFER
	ANL	A,#00000001B			;IS BUFFER EMPTY?
	JNZ	H11_RET
;------
MORE_MESSAGE:
	MOV	R0,#BYTE_TO_SEND
	MOV	A,@R0
	JZ	HUB_NO_DATA_TO_HOST
	MOV	DPH,MSG_DPH
	MOV	DPL,MSG_DPL
	ACALL	SEND_TO_HOST
MORE_MESSAGE_1:
	RET
HUB_NO_DATA_TO_HOST:
	ACALL	SEND_ZERO_PACKET
	RET


;********************************************************************
EMB_ED0_INPUT:
;----
	SETB	SEL_EPT_FLAG			    ;SELECT EMBED BUFFER
;----
	MOV	I2C_BUFFER,#READ_LAST_EMBF_IN_ST    ;READ ENDPOINT STATUS FROM H11
	ACALL	READ_ONE_BYTE
;------
	MOV	I2C_BUFFER,#SEL_EPT_EMBF_IN	;SELECT EMB EPT0 IN
	ACALL	READ_ONE_BYTE
	MOV	A,I2C_BUFFER
	ANL	A,#00000001B			;IS BUFFER EMPTY?
	JNZ	H11_RET
;------
EMB_MORE_MESSAGE:
	MOV	R0,#EMB_BYTE_TO_SEND
	MOV	A,@R0
	JZ	EMB_NO_DATA_TO_HOST
	MOV	DPH,EMB_MSG_DPH
	MOV	DPL,EMB_MSG_DPL
	ACALL	EMB_SEND_TO_HOST
EMB_END_IN:
H11_RET1:
	RET
EMB_NO_DATA_TO_HOST:
	ACALL	SEND_EMB_ZERO_PACKET
	RET

;********************************************************************
HUB_ED0_OUTPUT:
;------
	CLR	SEL_EPT_FLAG			    ;SELECT HUB BUFFER
;-----
	MOV	I2C_BUFFER,#READ_LAST_EPT0_OUT_ST    ;READ ENDPOINT STATUS FROM H11
	ACALL	READ_ONE_BYTE			;
;------
	MOV	I2C_BUFFER,#READ_EPT0_OUT_ST  ;SETUP PACKET STATUS
	ACALL	READ_ONE_BYTE
	MOV	A,I2C_BUFFER
	ANL	A,#00000100B
	JNZ	READ_SETUP_DATA 		;SETUP PACKET IS NOT ACK
	MOV	A,I2C_BUFFER
	ANL	A,#00000001B
	JNZ	H11_RET1			;OUT TOKEN COMING, NO SUPPORT
	AJMP	H11_RET1			;NO TOKEN COMING

;---------------------------			;MAYBE OUT TOKEN
READ_SETUP_DATA:
	MOV	I2C_BUFFER,#SEL_EPOINT_0_IN
	ACALL	WRITE_COMMAND
;-----
	MOV	I2C_BUFFER,#ACK_SETUP		;SEND ACK TO ENDPOINT 0 IN
	ACALL	WRITE_COMMAND
;---------------------------
	MOV	I2C_BUFFER,#SEL_EPOINT_0_OUT
	ACALL	WRITE_COMMAND
;-----
	MOV	I2C_BUFFER,#ACK_SETUP		;SEND ACK TO ENDPOINT 0 OUT
	ACALL	WRITE_COMMAND
;---------------------------
	MOV	I2C_BUFFER,#READ_BUFFER 	;SEND READ BUFFER CMD TO H11
	ACALL	READ_TEN_BYTE
;-----
	MOV	I2C_BUFFER,#CLEAR_BUF		;RELEASE BUFFER
	ACALL	WRITE_COMMAND
;-----
	MOV	A,I2C_BUFFER+1			;READ TOTAL HOW MANY BYTES
	CJNE	A,#8,H11_RET1			;NOT CORRECT COMMAND FROM HOST

;-----------------------------------------------------------------
; DECODE HUB ENDPOINT 0 STANDARD REQUEST COMMAND
;-----------------------------------------------------------------
DECODE_REQUEST:
;---
	MOV	R2,I2C_BUFFER+OFFSET_bmReqTyp
	MOV	R3,I2C_BUFFER+OFFSET_bmReq
	MOV	R4,I2C_BUFFER+OFFSET_wValue
	MOV	R5,I2C_BUFFER+OFFSET_wIndex
	MOV	R6,I2C_BUFFER+OFFSET_wLenght
;---
	MOV	A,R2			;I2C_BUFFER+OFFSET_bmReqTyp
	ANL	A,#01100000B
	JZ	STANDARD_REQUEST
	AJMP	CLASS_REQUEST
;---------
STANDARD_REQUEST:
	MOV	A,R3			;I2C_BUFFER+OFFSET_bmReq
	JZ	GET_STATUS_P	       ;0
	DEC	A
	JZ	CLEAR_FEATURE_P 	;1
	DEC	A			;2  UNDEFINED
	DEC	A
	JZ	SET_FEATURE_P		;3
	DEC	A			;4  UNDEFINED
	DEC	A
	JZ	SET_ADDRESS		;5
	DEC	A
	JZ	GET_DESCRIPTOR_P	;6
	DEC	A			;7 SET_DESCRIPTOR, NO SUPPORT
	DEC	A
	JZ	GET_CONFIG		;8
	DEC	A
	JZ	SET_CONFIG		;9
	DEC	A
	JZ	GET_INTERFACE		;A
	DEC	A
	JZ	SET_INTERFACE		;B
;-----
NO_SUPPORT_REQ:
	ACALL	SEND_EP0I_STALL 	;10 - GET INTERFACE, NO SUUPORT
	RET				;11 - SET INTERFACE, NO SUPPORT
					;12 - SYNCH FRAME, NO SUPPORT
;------------------------------------
CLEAR_FEATURE_P:
	AJMP	CLEAR_FEATURE
SET_FEATURE_P:				;3
	AJMP	SET_FEATURE
GET_DESCRIPTOR_P:			;6
	AJMP	GET_DESCRIPTOR
GET_STATUS_P:
	AJMP	GET_STATUS

;=================================================================
SET_INTERFACE:
	ACALL	SEND_ZERO_PACKET
	RET
;-----
GET_INTERFACE:
	MOV	I2C_BUFFER,#WRITE_BUFFER	    ;WRITE BUFFER COMMAND
	MOV	I2C_BUFFER+1,#0
	MOV	I2C_BUFFER+2,#1
	MOV	I2C_BUFFER+3,#0
	ACALL	WRITE_THREE_BYTE
	RET
;------
;=================================================================
SET_ADDRESS:				;5
	MOV	R0,#I2C_BUFFER+OFFSET_wValue
	MOV	A,@R0
	ORL	A,#080H
	MOV	HUB_ADDR,A
;---
	ACALL	SEND_ZERO_PACKET
;----
SET_ADDRESS_LOOP:
	JB	H11_INT_PORT,SET_ADDRESS_LOOP	;IF H11 INT PORT HI THEN RETURN
;---
SET_HUB_ADDR:
	MOV	I2C_BUFFER,#SET_HUB_ADDRESS_EN	;WRITE NEW ADDRESS TO HUB
	MOV	I2C_BUFFER+1,HUB_ADDR
	ACALL	WRITE_ONE_BYTE
;----
	MOV	I2C_BUFFER,#READ_LAST_EPT0_IN_ST ;CLEAR HUB EP0 IN SETTING OF
	ACALL	READ_ONE_BYTE			 ;INTERRUPT REGISTER
	RET

;===============================================================
GET_CONFIG:				;8
;---
	MOV	I2C_BUFFER,#WRITE_BUFFER	    ;WRITE BUFFER COMMAND
	MOV	I2C_BUFFER+1,#0
	MOV	I2C_BUFFER+2,#1
	MOV	I2C_BUFFER+3,#0
	JB	HUB_CONFIG,HUB_HAD_CONFIG
GET_CONFIG_1:
	ACALL	WRITE_THREE_BYTE
	RET
HUB_HAD_CONFIG:
	MOV	I2C_BUFFER+3,#1
	AJMP	GET_CONFIG_1


;==================================================================
SET_CONFIG:				;9
	MOV	A,R4
	JZ	DISABLE_CONFIG
	CJNE	A,#1,NO_SUPPORT_REQ	;WRONG_SET_CONFIG
;-----
	MOV	I2C_BUFFER,#SET_MODE		;SET STOP PLL DURING
	MOV	I2C_BUFFER+1,#10110000B 	;SUSPEND MODE
	ACALL	WRITE_ONE_BYTE
;-----
	SETB	HUB_CONFIG
	MOV	I2C_BUFFER,#SET_EPOINT_EN
	MOV	I2C_BUFFER+1,#01H
SET_CONFIG_1:
	ACALL	WRITE_ONE_BYTE
;---
	ACALL	SEND_ZERO_PACKET
	RET
;----
DISABLE_CONFIG:
	CLR	HUB_CONFIG
	MOV	I2C_BUFFER,#SET_EPOINT_EN
	MOV	I2C_BUFFER+1,#00H
	AJMP	SET_CONFIG_1


;=================================================================
; GET HUB OR PORT STATUS FROM H11
;------------------------------------
GET_STATUS:			;0
;-----
	MOV	I2C_BUFFER,#WRITE_BUFFER
	MOV	I2C_BUFFER+1,#0
	MOV	I2C_BUFFER+2,#2
	MOV	I2C_BUFFER+3,#0
	MOV	I2C_BUFFER+4,#0
;-----
	MOV	A,R2			;I2C_BUFFER+OFFSET_bmReqTyp
	ANL	A,#00000011B
	JZ	GET_DEVICE_STATUS
	DEC	A
	JZ	GET_IF_STATUS
	DEC	A
	JZ	GET_EPT_STATUS
GET_WRONG_RECIPIENT:
	AJMP	NO_SUPPORT_REQ
;-----
GET_DEVICE_STATUS:
	MOV	A,HUB_STATUS
	ANL	A,#00000011B
	MOV	I2C_BUFFER+3,A		;STATUS OF SELF POWER AND REMOTE WAKEUP
GET_IF_STATUS:
HUB_EPT0_OUT:
EPT_UN_STALL:
	ACALL	WRITE_FOUR_BYTE 	;WRITE FOUR BYTES TO H11
	RET
;-----
GET_EPT_STATUS:
	MOV	A,R5			;I2C_BUFFER+OFFSET_wIndex
	JZ	HUB_EPT0_OUT		;SHOULD BE UN_STALL
;---
	XRL	A,#080H
	JZ	HUB_EPT0_IN		;HUB EPT0 IN
;---
	MOV	A,R5
	XRL	A,#081H
	JZ	HUB_EPT1_IN		;HUB EPT0 OUT
	AJMP	NO_SUPPORT_REQ
;-----
HUB_EPT0_IN:
	JNB	EPT0_OUT_STALL,EPT_UN_STALL	;HUB EPT0 IN UN_STALL
;-----
EPT_STALL:
	MOV	I2C_BUFFER+3,#1 		;HUB EPT0 IN STALL
	AJMP	EPT_UN_STALL
;-----
HUB_EPT1_IN:
	JNB	EPT1_IN_STALL,EPT_UN_STALL	;HUB EPT1 IN UN_STALL
	AJMP	EPT_STALL			;HUB EPT1 IN STALL

;==================================================================
; CLEAR DEVICE,INTERFACE,ENDPOINT FEATURE
;----------------------------------------
CLEAR_FEATURE:				;1
	MOV	A,R2			;I2C_BUFFER+OFFSET_bmReqTyp
	JZ	CLEAR_DEVICE_FEATURE
	DEC	A
	JZ	CLEAR_INTERFACE_FEATURE
	DEC	A
	JZ	CLEAR_EPT_FEATURE
;-----
CLEAR_INTERFACE_FEATURE:
	AJMP	NO_SUPPORT_REQ
;----
CLEAR_DEVICE_FEATURE:
	MOV	I2C_BUFFER,#SET_MODE		;CLEAR REMOTE WAKE-UP
	MOV	I2C_BUFFER+1,#10110100B 	;FUNCTION
	ACALL	WRITE_ONE_BYTE
;----
	CLR	REMOTE_WAKE
;-----
CLEAR_FEATURE_RET:
CLR_EPT0_OUT:
	ACALL	SEND_ZERO_PACKET
	RET

;---------------------------------	;CLEAR ENDPOINR FEATURE
CLEAR_EPT_FEATURE:
	MOV	A,R5			;I2C_BUFFER+OFFSET_wIndex
	JZ	CLR_EPT0_OUT		;SHOULD BE UN_STALL
;---
	XRL	A,#080H
	JZ	CLR_EPT0_IN		;HUB EPT0 IN
;---
	MOV	A,R5
	XRL	A,#081H
	JZ	CLR_EPT1_IN		;HUB EPT0 OUT
	AJMP	NO_SUPPORT_REQ
;----------------
CLR_EPT0_IN:
	CLR	EPT0_IN_STALL		;HUB EPT0 IN STALL FLAG
	MOV	I2C_BUFFER,#SET_EPT0_IN_ST	;SET ENDPOINT STATUS
	MOV	I2C_BUFFER+1,#0 	;CLEAR ENDPOINT STALL CONDITION
	ACALL	WRITE_ONE_BYTE
	AJMP	CLEAR_FEATURE_RET
;---------------
CLR_EPT1_IN:
	CLR	EPT1_IN_STALL		;HUB EPT1 IN STALL FLAG
	AJMP	CLEAR_FEATURE_RET

;=========================================================
SET_FEATURE:				;3
	MOV	A,R2			;I2C_BUFFER+OFFSET_bmReqTyp
	JZ	SET_DEVICE_FEATURE
	DEC	A
	JZ	SET_INTERFACE_FEATURE
	DEC	A
	JZ	SET_EPT_FEATURE
;-----
SET_INTERFACE_FEATURE:
NO_SUPPORT_REQ1:
	AJMP	NO_SUPPORT_REQ
;----
SET_DEVICE_FEATURE:
	MOV	I2C_BUFFER,#SET_MODE		;ENABLE REMOTE WAKE-UP
	MOV	I2C_BUFFER+1,#10110101B 	;
	ACALL	WRITE_ONE_BYTE
;-----
	SETB	REMOTE_WAKE
;-----
SET_FEATURE_RET:
SET_EPT0_OUT:
	ACALL	SEND_ZERO_PACKET
	RET

;---------------------------------	;CLEAR ENDPOINR FEATURE
SET_EPT_FEATURE:
	MOV	A,R5			;I2C_BUFFER+OFFSET_wIndex
	JZ	SET_EPT0_OUT		;SHOULD BE UN_STALL
;---
	XRL	A,#080H
	JZ	SET_EPT0_IN		;HUB EPT0 IN
;---
	MOV	A,R5
	XRL	A,#081H
	JZ	SET_EPT1_IN		;HUB EPT0 OUT
	AJMP	NO_SUPPORT_REQ
;----------------
SET_EPT0_IN:
	SETB	EPT0_IN_STALL		;HUB EPT0 IN STALL FLAG
	MOV	I2C_BUFFER,#SET_EPT0_IN_ST	;SET ENDPOINT STATUS
	MOV	I2C_BUFFER+1,#1 	;CLEAR ENDPOINT STALL CONDITION
	ACALL	WRITE_ONE_BYTE
	AJMP	SET_FEATURE_RET
;---------------
SET_EPT1_IN:
	SETB	EPT1_IN_STALL		;HUB EPT1 IN STALL FLAG
	AJMP	SET_FEATURE_RET
;-------


;=======================================================
GET_DESCRIPTOR: 			;6
	MOV	A,I2C_BUFFER+OFFSET_wValue+1	;I2C_BUFFER+OFFSET_wValue+1
;----
CHK_DEV_DESC:
	CJNE	A,#01,CHK_CONF_DESC
	MOV	DPTR,#STAND_DEV_DESC
	MOV	R0,#I2C_BUFFER+OFFSET_wLenght
;---
	INC	R0			;IF HOST ISSUE OVER 256 BYTES
	MOV	A,@R0
	JNZ	CHK_DEV_DESC1
	DEC	R0
;---
	MOV	A,@R0
	CLR	C
	SUBB	A,#18
	JC	SEND_DESCRIPTOR
CHK_DEV_DESC1:
	MOV	@R0,#18
	AJMP	SEND_DESCRIPTOR
;----
CHK_CONF_DESC:
	CJNE	A,#02,NO_SUPPORT_REQ2
	SETB	PWR_SRC_FLAG		;BUS /SELF POWER FLAG
	MOV	DPTR,#CONFIG_DESC
	MOV	R0,#I2C_BUFFER+OFFSET_wLenght
;---
	INC	R0			;IF HOST ISSUE OVER 256 BYTES
	MOV	A,@R0
	JNZ	CHK_CONF_DESC1
;--
	DEC	R0
	MOV	A,@R0
	CLR	C
	SUBB	A,#25
	JC	SEND_DESCRIPTOR
CHK_CONF_DESC1:
	MOV	@R0,#25
;----
SEND_DESCRIPTOR:
	ACALL	SEND_TO_HOST
	RET




;*******************************************************
;	MOV	R2,I2C_BUFFER+OFFSET_bmReqTyp
;	MOV	R3,I2C_BUFFER+OFFSET_bmReq
;	MOV	R4,I2C_BUFFER+OFFSET_wValue
;	MOV	R5,I2C_BUFFER+OFFSET_wIndex
;	MOV	R6,I2C_BUFFER+OFFSET_wLenght
;*******************************************************
CLASS_REQUEST:
	MOV	A,R3			;I2C_BUFFER+OFFSET_bmReq
	JZ	GET_C_STATUS		;0
	DEC	A
	JZ	CLEAR_C_FEATURE_P	  ;1
	DEC	A
;;;	JZ	GET_C_STATE		;2  NO SUPPORT
	DEC	A
	JZ	SET_C_FEATURE_P 	;3
	DEC	A			;4
	DEC	A			;5
	DEC	A
	JZ	GET_C_DESCRIPTOR	;6
;;	DEC	A
;;	JZ	SET_C_DESCRIPTOR	;7 NO SUPPORT
;--------
NO_SUPPORT_REQ2:
	AJMP	NO_SUPPORT_REQ

SET_C_FEATURE_P:
	AJMP	SET_C_FEATURE
CLEAR_C_FEATURE_P:
	AJMP	CLEAR_C_FEATURE 	  ;1

;=========================================
GET_C_DESCRIPTOR:			;6
	JB	H12_FLAG,HUB_DESC_TYPE_H12
	MOV	DPTR,#HUB_DESC
HUB_TYPE:
	MOV	R0,#I2C_BUFFER+OFFSET_wLenght
	MOV	A,@R0
	CLR	C
	SUBB	A,#9
	JC	SEND_HUB_DESCRIPTOR
	MOV	@R0,#9
	AJMP	SEND_HUB_DESCRIPTOR
SEND_HUB_DESCRIPTOR:
	ACALL	SEND_TO_HOST
	RET
HUB_DESC_TYPE_H12:
	MOV	DPTR,#HUB_DESC_H12
	AJMP	HUB_TYPE


;=========================================
GET_C_STATUS:				;0 GET CLASS STATUS
	MOV	A,R5
	JZ	GET_HUB_STATUS
	DEC	A
	JZ	GET_PORT1_STATUS
	DEC	A
	JZ	GET_PORT2_STATUS
	DEC	A
	JZ	GET_PORT3_STATUS
	DEC	A
	JZ	GET_PORT4_STATUS
	DEC	A
	JZ	GET_PORT5_STATUS
NO_SUPPORT_REQ3:
	AJMP	NO_SUPPORT_REQ
;----------------------------------------
GET_HUB_STATUS:
	MOV	I2C_BUFFER,#0E0H
	ACALL	READ_TWO_BYTE
;----
	MOV	A,I2C_BUFFER+1		;SECOND BYTE
	RR	A
	RR	A
	ANL	A,#00000010B
	MOV	R6,A
	MOV	R7,A
	AJMP	SEND_STATUS_HOST
GET_PORT1_STATUS:
;---
	MOV	I2C_BUFFER,#SET_STATUS_CHG_BITS ;CLEAR HUB ENDPOINT1 INTERRUPT
	MOV	I2C_BUFFER+1,#00		;TRS FOR PORT0
	ACALL	WRITE_ONE_BYTE
;---
	MOV	R6,EMBED_STATUS0
	MOV	R7,EMBED_STATUS1
	AJMP	SEND_STATUS_HOST
;-----
GET_PORT2_STATUS:
GET_PORT3_STATUS:
GET_PORT4_STATUS:
GET_PORT5_STATUS:
	MOV	A,R5
	DEC	A
	DEC	A
	ORL	A,#0E0H
	MOV	I2C_BUFFER,A
	ACALL	READ_TWO_BYTE
;---
	MOV	R6,I2C_BUFFER		    ;FIRST BYTE
	MOV	R7,I2C_BUFFER+1 	    ;SECOND BYTE
;---
SEND_STATUS_HOST:
;---
	MOV	I2C_BUFFER,#WRITE_BUFFER
	MOV	I2C_BUFFER+1,#0
	MOV	I2C_BUFFER+2,#4
;---
	MOV	A,R6
	ANL	A,#00011111B
	MOV	I2C_BUFFER+3,A
;---
	MOV	A,R6
	SWAP	A
	RR	A
	ANL	A,#00000011B
	MOV	I2C_BUFFER+4,A
;---
	MOV	A,R7
	ANL	A,#00011111B
	MOV	I2C_BUFFER+5,A
;----
	MOV	I2C_BUFFER+6,#0
;----
	ACALL	WRITE_SIX_BYTE
	RET
;======================================
CLEAR_C_FEATURE:			;1 GET CLASS FEATURE
	JNB	HUB_CONFIG,NO_SUPPORT_REQ3 ;HUB NOT CONFIGURATION,SEND STALL
;---
	MOV	A,R5			;
	JZ	CLEAR_HUB_FEATURE	;CLEAR HUB FEATURE
	DEC	A
	JNZ	CLEAR_DN_PORT
	AJMP	CLEAR_PORT0_FEATURE	 ;CLEAR EMBED PORT FEATURE
;------
CLEAR_DN_PORT:
	DEC	A
	MOV	R1,A
	ANL	A,#11111100B
	JNZ	NO_SUPPORT_REQ3 	;OVER PORT5, NO SUPPORT
;---
	MOV	A,R1
	ORL	A,#0E0H 		;CLEAR PORT2-5 FEATURE
	MOV	R1,A			;E0-E3
;-----
CLEAR_PORT_FEATURE:
	MOV	A,R4
;;	JZ				;0 PORT CONNECTION
	DEC	A
	JZ	CL_PORT_ENABLE		;1
	DEC	A
	JZ	CL_PORT_SUSPEND 	;2
	CLR	C			;3,4
	SUBB	A,#6
	JZ	CL_PORT_POWER		;8 CLEAR PORT POWER
	CLR	C
	SUBB	A,#8
	JZ	CL_C_PORT_CONN		;16
	DEC	A
	JZ	CL_C_PORT_ENABLE	;17
	DEC	A
	JZ	CL_C_PORT_SUSPEND	;18
	DEC	A
	JZ	CL_C_PORT_OVER_CURR	;19
	DEC	A
	JZ	CL_C_PORT_RESET 	;20
NO_SUPPORT_REQ4:
	AJMP	NO_SUPPORT_REQ
;-------------------------------------------
CL_PORT_ENABLE:
	MOV	R6,#F_PORT_ENABLE	;1
	AJMP	VALID_CL_FEATURE
;-----------
CL_PORT_SUSPEND:
	MOV	R6,#F_PORT_SUSPEND	;2
	AJMP	VALID_CL_FEATURE
;----------
CL_PORT_POWER:
	MOV	EMBED_STATUS0,#0	;8 ALL EMBED STATUS SHOULD BE CLEAR
	MOV	EMBED_STATUS1,#0
	CLR	E_PORT_POWER
;---
	MOV	R6,#F_PORT_POWER	;8
	AJMP	VALID_CL_FEATURE
;----------
CL_C_PORT_CONN:
	MOV	R6,#C_PORT_CONN 	;16
	AJMP	VALID_CL_FEATURE
;----------
CL_C_PORT_ENABLE:
	MOV	R6,#C_PORT_ENABLE	;17
	AJMP	VALID_CL_FEATURE
;-----------
CL_C_PORT_SUSPEND:
	MOV	R6,#C_PORT_SUSPEND	;18
	AJMP	VALID_CL_FEATURE
;-----------
CL_C_PORT_OVER_CURR:
	MOV	R6,#C_PORT_OVERCUR	;19
	AJMP	VALID_CL_FEATURE
;-----------
CL_C_PORT_RESET:
	MOV	R6,#FC_PORT_RESET	;20
;-----
VALID_CL_FEATURE:
	MOV	I2C_BUFFER,R1
	MOV	I2C_BUFFER+1,R6
	ACALL	WRITE_ONE_BYTE
;---
	ACALL	SEND_ZERO_PACKET
CL_RET:
	RET
;---------------
CLEAR_HUB_FEATURE:
	MOV	A,R4
	JB	ACC.0,CLEAR_HUB_OVERCURRENT ;NO SUPPORT FEATURE SELECTORS
	AJMP	NO_SUPPORT_REQ4
CLEAR_HUB_OVERCURRENT:
	MOV	R1,#0E0H		 ;CLEAR PORT2-5 OVERCURRENT
	AJMP	CL_C_PORT_OVER_CURR
;===================----------------
CLEAR_PORT0_FEATURE:
	MOV	A,R4
;;	JZ				;0 PORT CONNECTION
	DEC	A
	JZ	CL_PORT0_ENABLE 	;IGNORE 1:CL_PORT0_ENABLE
	DEC	A
	JZ	CL_PORT0_SUSPEND	;IGNORE 2:CL_PORT0_SUSPEND
	CLR	C			;3,4
	SUBB	A,#6
	JZ	CL_PORT0_POWER		;IGNORE 8:CLEAR PORT POWER
	CLR	C
	SUBB	A,#8
	JZ	CL_PORT0_OK		;IGNORE 16:CL_C_PORT0_CONN
	DEC	A
	JZ	CL_PORT0_C_ENABLE	;IGNORE 17:CL_C_PORT0_ENABLE
	DEC	A
	JZ	CL_PORT0_C_SUSPEND	;IGNORE 18:CL_C_PORT0_SUSPEND
	DEC	A
	JZ	CL_PORT0_OK		;IGNORE 19:CL_C_PORT0_OVER_CURR
	DEC	A
	JZ	CL_PORT0_RESET		;IGNORE 20:CL_C_PORT0_RESET
	AJMP	NO_SUPPORT_REQ
;------------------------------
CL_PORT0_POWER:
	MOV	I2C_BUFFER,#SET_EMB_ADDRESS_EN	;DISABLE EMBED ADDRESS
	MOV	I2C_BUFFER+1,#00
	ACALL	WRITE_ONE_BYTE
;----
	MOV	I2C_BUFFER,#SET_STATUS_CHG_BITS ;INFORM HOST, CURRENT EMBEDDED
	MOV	I2C_BUFFER+1,#02		;FUNCTION IS LOSE
	ACALL	WRITE_ONE_BYTE
;----
	MOV	R1,#0E0H
	AJMP	CL_PORT_POWER		;CLEAR ALL DN PORT POWER
;---------
CL_PORT0_SUSPEND:			;IGNORE 2:CL_PORT0_SUSPEND
	CLR	E_PORT_SUSPEND
	SETB	E_C_PORT_SUSPEND
;----
	MOV	I2C_BUFFER,#SET_STATUS_CHG_BITS ;INFORM HOST, CURRENT EMBEDDED
	MOV	I2C_BUFFER+1,#02		;FUNCTION IS LOSE
	ACALL	WRITE_ONE_BYTE
;----
	AJMP	CL_PORT0_OK

CL_PORT0_RESET:
	CLR	E_C_PORT_RESET
	ACALL	SEND_ZERO_PACKET
	LCALL	INIT_DDC2B
CC_RET:
	RET
;----
CL_PORT0_C_ENABLE:
	CLR	E_C_PORT_ENABLE
	AJMP	CL_PORT0_OK
;----
CL_PORT0_C_SUSPEND:
	CLR	E_C_PORT_SUSPEND
	AJMP	CL_PORT0_OK
;----
CL_PORT0_ENABLE:
	JNB	E_PORT_ENABLE,CL_PORT0_OK      ;PORT0 CURRENT IS DISBALE
	SETB	E_C_PORT_ENABLE   ;PORT0 CURRENT IS ENABLE, SET CHNAGE BIT
	CLR	E_PORT_ENABLE	  ;CLEAR PROT0 ENABLE
;---
	MOV	I2C_BUFFER,#SET_EMB_ADDRESS_EN	;DISABLE EMBED ADDRESS
	MOV	I2C_BUFFER+1,#00
	ACALL	WRITE_ONE_BYTE
;---
CL_PORT0_OK:
	ACALL	SEND_ZERO_PACKET
	RET


;==========================================
SET_C_FEATURE:				;3
	JNB	HUB_CONFIG,NO_SUPPORT_REQ5  ;HUB NOT CONFIGURATION
;-----
	MOV	A,R5			;I2C_BUFFER+OFFSET_wIndex
	JZ	NO_SUPPORT_REQ5 	;SET HUB FEATURE
	DEC	A
	JZ	SET_PORT0_FEATURE	;SET EMBEDED PORT FEATURE
SET_PORT_FEATURE:
	DEC	A
	MOV	R1,A			;PORT2-5
	ANL	A,#11111100B
	JNZ	NO_SUPPORT_REQ5 	;OVER PORT 5
	MOV	A,R1
	ORL	A,#0E8H
	MOV	R1,A			;E8,9,A,B
;-----
SET_FEATURE_TABLE:
	MOV	A,R4
;;	JZ				;0 PORT CONNECTION
	DEC	A
	JZ	SET_PORT_ENABLE 	;1
	DEC	A
	JZ	SET_PORT_SUSPEND	;2
	DEC	A			;3
	DEC	A
	JZ	SET_PORT_RESET		;4
	DEC	A			;5
	DEC	A			;6
	DEC	A			;7
	DEC	A
	JZ	SET_PORT_POWER		;8 CLEAR PORT POWER
NO_SUPPORT_REQ5:
	AJMP	NO_SUPPORT_REQ
;-------------------------------------------
SET_PORT_ENABLE:
	MOV	R6,#F_PORT_ENABLE	;1
	AJMP	VALID_SET_FEATURE
;--------------------
SET_PORT_SUSPEND:
	MOV	R6,#F_PORT_SUSPEND	;2
	AJMP	VALID_SET_FEATURE
;------------------
SET_PORT_RESET:
	MOV	R6,#FC_PORT_RESET	;4
VALID_SET_FEATURE:
	AJMP	VALID_CL_FEATURE
;------------------
SET_PORT_POWER:
	SETB	E_PORT_POWER	;WHEN ONE PORT HAS POWER TEHN EVERY ONE HAS POWER
	SETB	E_PORT_CONN	;THE PORT0 IS ALWAYS CONNECTED TO HUB
;----
	MOV	R1,#0E8H		;turn on port2 power as well as OV detection
	ACALL	SET_POWER_ON
	MOV	R1,#0E9H		;turn on port3 power as well as OV detection
	ACALL	SET_POWER_ON
	MOV	R1,#0EAH		;turn on port4 power as well as OV detection
	ACALL	SET_POWER_ON
	MOV	R1,#0EBH		;turn on port5 power as well as OV detection
	ACALL	SET_POWER_ON
;----
	ACALL	SEND_ZERO_PACKET
	RET
;----
SET_POWER_ON:
	MOV	I2C_BUFFER,R1			;TURN ON POWER
	MOV	I2C_BUFFER+1,#F_PORT_POWER
	ACALL	WRITE_ONE_BYTE
	NOP
	NOP
	MOV	I2C_BUFFER,R1			;TURN ON OVERCURRENT CIRCUIT
	MOV	I2C_BUFFER+1,#F_PORT_POWER
	ACALL	WRITE_ONE_BYTE
	RET
;======================-----------------
SET_PORT0_FEATURE:
	MOV	A,R4
;;;	JZ				;0 PORT CONNECTION
	DEC	A
	JZ	SET_PORT0_ENABLE	;IGNORE  1:SET_PORT_ENABLE
	DEC	A
	JZ	SET_PORT0_SUSPEND	;IGNORE  2:SET_PORT_SUSPEND
	DEC	A			;3
	DEC	A
	JZ	SET_PORT0_RESET 	;IGNORE  4:SET_PORT0_RESET
	DEC	A			;5
	DEC	A			;6
	DEC	A			;7
	DEC	A
	JZ	SET_PORT0_POWER 	;8 SET PORT POWER
	AJMP	NO_SUPPORT_REQ
;-------------------------------------------
SET_PORT0_POWER:
;----
	CLR	INIT_FLAG
;----
	MOV	I2C_BUFFER,#SET_STATUS_CHG_BITS ;INFORM HOST, CURRENT EMBEDDED
	MOV	I2C_BUFFER+1,#02		;FUNCTION IS ATTACHED
	ACALL	WRITE_ONE_BYTE
;----
	AJMP	SET_PORT_POWER
;----
SET_PORT0_SUSPEND:			 ;IGNORE 2:CL_PORT0_SUSPEND
	SETB	E_PORT_SUSPEND
;---
SET_PORT0_OK:
	ACALL	SEND_ZERO_PACKET
H11_RET2:
	RET

;---------------------
SET_PORT0_RESET:
	JNB	E_PORT_POWER,SET_PORT0_OK	;PORT0 NO POWER
;--------------
	MOV	I2C_BUFFER,#SET_STATUS_CHG_BITS ;INFORM HOST, CURRENT EMBEDDED
	MOV	I2C_BUFFER+1,#02		;FUNCTION IS LOSE
	ACALL	WRITE_ONE_BYTE
;-------------
	SETB	E_C_PORT_RESET			;SETTING PORT RESET READY FLAG
	CLR	E_PORT_SUSPEND
	MOV	EMB_ADDR,#0
	AJMP	SET_PORT0_1

;---------------------
SET_PORT0_ENABLE:
	JNB	E_PORT_POWER,SET_PORT0_OK	;PORT0 NO POWER
	JB	E_PORT_ENABLE,SET_PORT0_OK	;CURRENT IS ENABLE
	SETB	E_C_PORT_ENABLE 		;CURRENT IS DISBALE
;---
SET_PORT0_1:
	SETB	E_PORT_ENABLE			;SET PROT0 ENABLE
;----
	MOV	I2C_BUFFER,#SET_EMB_ADDRESS_EN	;DISABLE EMBED ADDRESS
	MOV	A,EMB_ADDR			;ADDRESS OF THE EMBEDDED FUNCTION
	ORL	A,#080H
	MOV	I2C_BUFFER+1,A
	ACALL	WRITE_ONE_BYTE
;----
	MOV	I2C_BUFFER,#SET_EPOINT_EN	;ENABLE HUB INTERRUPT ENDPOINT
	MOV	I2C_BUFFER+1,#03H		;and EMBEDDED FUNCTION
	ACALL	WRITE_ONE_BYTE
;---
	AJMP	SET_PORT0_OK
;*******************************************
CHK_LAST_COMM:
	MOV	A,EMB_LAST_COM
	CJNE	A,#REQ_SET_FEATURE,WRONG_REQ_COMMAND   ;LAST COM IS NOT SET_CURRENT.
	MOV	EMB_LAST_COM,#0
;----
	MOV	I2C_BUFFER,#SEL_EPT_EMBF_OUT
	ACALL	WRITE_COMMAND
;-----
	MOV	I2C_BUFFER,#READ_BUFFER 	;SEND READ BUFFER CMD TO H11
	ACALL	READ_TEN_BYTE
;------
	MOV	I2C_BUFFER,#CLEAR_BUF		;RELEASE BUFFER
	ACALL	WRITE_COMMAND
;-----
	MOV	A,I2C_BUFFER+1			;READ TOTAL HOW MANY BYTES
	CJNE	A,#3,EMB_DATA_ERROR		;NOT CORRECT COMMAND FROM HOST
;-----
	LCALL	SET_DAC_DATA
	ACALL	SEND_EMB_ZERO_PACKET
	RET

;------
WRONG_REQ_COMMAND:
	MOV	I2C_BUFFER,#CLEAR_BUF		;RELEASE BUFFER
	ACALL	WRITE_COMMAND
EMB_DATA_ERROR:
	AJMP	H11_RET2
;********************************************************************
EMB_ED0_OUTPUT:
;---
	SETB	SEL_EPT_FLAG		    ;SELECT EMBED BUFFER
;---
	MOV	I2C_BUFFER,#READ_LAST_EMBF_OUT_ST    ;READ ENDPOINT STATUS FROM H11
	ACALL	READ_ONE_BYTE			;
;----
	MOV	I2C_BUFFER,#READ_EMBF_OUT_ST	      ;SETUP PACKET STATUS
	ACALL	READ_ONE_BYTE
	MOV	A,I2C_BUFFER
	ANL	A,#00000100B
	JNZ	EMB_READ_SETUP_DATA		    ;SETUP PACKET IS NOT ACK
	MOV	A,I2C_BUFFER
	ANL	A,#00100000B
	JNZ	CHK_LAST_COMM			;OUT TOKEN COMING, BUFFER FULL
	AJMP	EMB_DATA_ERROR			;NO TOKEN COMING

;---------------------------			;MAYBE OUT TOKEN
EMB_READ_SETUP_DATA:
	MOV	I2C_BUFFER,#SEL_EPT_EMBF_IN
	ACALL	WRITE_COMMAND
;-----
	MOV	I2C_BUFFER,#ACK_SETUP		;SEND ACK TO ENDPOINT 0 IN
	ACALL	WRITE_COMMAND
;---------------------------
	MOV	I2C_BUFFER,#SEL_EPT_EMBF_OUT
	ACALL	WRITE_COMMAND
;-----
	MOV	I2C_BUFFER,#ACK_SETUP		;SEND ACK TO ENDPOINT 0 OUT
	ACALL	WRITE_COMMAND
;---------------------------
	MOV	I2C_BUFFER,#READ_BUFFER 	;SEND READ BUFFER CMD TO H11
	ACALL	READ_TEN_BYTE
;------
	MOV	I2C_BUFFER,#CLEAR_BUF		;RELEASE BUFFER
	ACALL	WRITE_COMMAND
;-----
	MOV	A,I2C_BUFFER+1			;READ TOTAL HOW MANY BYTES
	CJNE	A,#8,EMB_DATA_ERROR		;NOT CORRECT COMMAND FROM HOST
;-----------------------------------------------------------------
; DECODE HUB ENDPOINT 0 STANDARD REQUEST COMMAND
;-----------------------------------------------------------------
EMB_DECODE_REQUEST:
;---
	MOV	R2,I2C_BUFFER+OFFSET_bmReqTyp
	MOV	R3,I2C_BUFFER+OFFSET_bmReq
	MOV	R4,I2C_BUFFER+OFFSET_wValue
	MOV	R5,I2C_BUFFER+OFFSET_wIndex
	MOV	R6,I2C_BUFFER+OFFSET_wLenght
;---
	MOV	A,R2			;I2C_BUFFER+OFFSET_bmReqTyp
	ANL	A,#01100000B
	JZ	EMB_STANDARD_REQUEST
	AJMP	EMB_CLASS_REQUEST
;---------
EMB_STANDARD_REQUEST:
	MOV	A,R3			;I2C_BUFFER+OFFSET_bmReq
	JZ	EMB_GET_STATUS_P	;0
	DEC	A
	JZ	EMB_CLEAR_FEATURE_P	;1
	DEC	A			;2  UNDEFINED
	DEC	A
	JZ	EMB_SET_FEATURE_P	;3
	DEC	A			;4  UNDEFINED
	DEC	A
	JZ	EMB_SET_ADDRESS 	;5
	DEC	A
	JZ	EMB_GET_DESCRIPTOR_P	;6
	DEC	A			;7 SET_DESCRIPTOR, NO SUPPORT
	DEC	A
	JZ	EMB_GET_CONFIG		;8
	DEC	A
	JZ	EMB_SET_CONFIG		;9
	DEC	A
	JZ	EMB_GET_INTERFACE	;A
	DEC	A
	JZ	EMB_SET_INTERFACE	;B
;-----
EMB_NO_SUPPORT_REQ:
	ACALL	SEND_EMB0I_STALL	;10 - GET INTERFACE, NO SUUPORT
	RET				;11 - SET INTERFACE, NO SUPPORT
					;12 - SYNCH FRAME, NO SUPPORT
;------------------------------------
EMB_CLEAR_FEATURE_P:
	AJMP	EMB_CLEAR_FEATURE
EMB_SET_FEATURE_P:			;3
	AJMP	EMB_SET_FEATURE
EMB_GET_DESCRIPTOR_P:			;6
	AJMP	EMB_GET_DESCRIPTOR
EMB_GET_STATUS_P:	;0
	AJMP	EMB_GET_STATUS		;0

;=================================================================
EMB_SET_INTERFACE:
	ACALL	SEND_EMB_ZERO_PACKET
	RET
;-----
EMB_GET_INTERFACE:
	MOV	I2C_BUFFER,#WRITE_BUFFER	    ;WRITE BUFFER COMMAND
	MOV	I2C_BUFFER+1,#0
	MOV	I2C_BUFFER+2,#1
	MOV	I2C_BUFFER+3,#0
	ACALL	WRITE_THREE_BYTE
	RET
;------
;=================================================================
EMB_SET_ADDRESS:
	MOV	A,R4
	ORL	A,#080H
	MOV	EMB_ADDR,A		;I2C_BUFFER+OFFSET_wValue
;---
	ACALL	SEND_EMB_ZERO_PACKET
;----
EMB_LOOP:
	MOV	I2C_BUFFER,#READ_INT		;READ INTERRUPT REGISTER
	ACALL	READ_ONE_BYTE			;READ DATA FROM H11
;------
	MOV	A,I2C_BUFFER
	JNB	ACC.3,EMB_LOOP
;---
SET_EMB_ADDR:
	MOV	I2C_BUFFER,#SET_EMB_ADDRESS_EN	;WRITE NEW ADDRESS TO HUB
	MOV	I2C_BUFFER+1,EMB_ADDR
	ACALL	WRITE_ONE_BYTE
;----
	MOV	I2C_BUFFER,#READ_LAST_EMBF_IN_ST    ;READ ENDPOINT STATUS FROM H11
	ACALL	READ_ONE_BYTE			 ;CLEAR HUB ED0 IN FLAG
	RET


;===============================================================
EMB_GET_CONFIG: 			    ;8
;---
	MOV	I2C_BUFFER,#WRITE_BUFFER	    ;WRITE BUFFER COMMAND
	MOV	I2C_BUFFER+1,#0
	MOV	I2C_BUFFER+2,#1
	MOV	I2C_BUFFER+3,#0
	JB	EMB_CONFIG,EMB_HAD_CONFIG
EMB_GET_CONFIG_1:
	ACALL	WRITE_THREE_BYTE
	RET
EMB_HAD_CONFIG:
	MOV	I2C_BUFFER+3,#1
	AJMP	EMB_GET_CONFIG_1


;==================================================================
EMB_SET_CONFIG: 			    ;9
	MOV	A,R4
	JZ	EMB_DISABLE_CONFIG
	CJNE	A,#1,EMB_NO_SUPPORT_REQ     ;WRONG_SET_CONFIG
;-----
	SETB	EMB_CONFIG
;----
	MOV	I2C_BUFFER,#SET_EPOINT_EN	;ENABLE function endpoint1
	MOV	I2C_BUFFER+1,#03h
	ACALL	WRITE_ONE_BYTE
;---
EMB_SET_CONFIG_1:
	ACALL	SEND_EMB_ZERO_PACKET
	RET
;----
EMB_DISABLE_CONFIG:
	CLR	EMB_CONFIG
	AJMP	EMB_SET_CONFIG_1


;=================================================================
; GET HUB OR PORT STATUS FROM H11
;------------------------------------
EMB_GET_STATUS: 		    ;0
;-----
	MOV	I2C_BUFFER,#WRITE_BUFFER
	MOV	I2C_BUFFER+1,#0
	MOV	I2C_BUFFER+2,#2
	MOV	I2C_BUFFER+3,#0
	MOV	I2C_BUFFER+4,#0
;-----
	MOV	A,R2			;I2C_BUFFER+OFFSET_bmReqTyp
	ANL	A,#00000011B
	JZ	EMB_GET_DEVICE_STATUS
	DEC	A
	JZ	EMB_GET_IF_STATUS
	DEC	A
	JZ	EMB_GET_EPT_STATUS
EMB_GET_WRONG_RECIPIENT:
	AJMP	EMB_NO_SUPPORT_REQ
;-----
EMB_GET_DEVICE_STATUS:
	MOV	A,EMB_STATUS
	ANL	A,#00000011B
	MOV	I2C_BUFFER+3,A		;STATUS OF SELF POWER AND REMOTE WAKEUP
EMB_GET_IF_STATUS:
EMB_EPT0_OUT:
EMB_EPT_UN_STALL:
	ACALL	WRITE_FOUR_BYTE 	;WRITE FOUR BYTES TO H11
	RET
;-----
EMB_GET_EPT_STATUS:
	MOV	A,R5			;I2C_BUFFER+OFFSET_wIndex
	JZ	EMB_EPT0_OUT		;SHOULD BE UN_STALL
;---
	XRL	A,#080H
	JZ	EMB_EPT0_IN		;HUB EPT0 IN
;---
	MOV	A,R5
	XRL	A,#081H
	JZ	EMB_EPT1_IN		;HUB EPT0 OUT
	AJMP	EMB_NO_SUPPORT_REQ
;-----
EMB_EPT0_IN:
	JNB	EMB_EPT0_OUT_STALL,EMB_EPT_UN_STALL ;HUB EPT0 IN UN_STALL
;-----
EMB_EPT_STALL:
	MOV	I2C_BUFFER+3,#1 		    ;EMBB EPT0 IN STALL
	AJMP	EMB_EPT_UN_STALL
;-----
EMB_EPT1_IN:
	JNB	EMB_EPT1_IN_STALL,EMB_EPT_UN_STALL  ;HUB EPT1 IN UN_STALL
	AJMP	EMB_EPT_STALL			    ;HUB EPT1 IN STALL

;==================================================================
; CLEAR DEVICE,INTERFACE,ENDPOINT FEATURE
;----------------------------------------
EMB_CLEAR_FEATURE:			;1
	MOV	A,R2			;I2C_BUFFER+OFFSET_bmReqTyp
	JZ	EMB_CLEAR_DEVICE_FEATURE
	DEC	A
	JZ	EMB_CLEAR_INTERFACE_FEATURE
	DEC	A
	JZ	EMB_CLEAR_EPT_FEATURE
;-----
EMB_CLEAR_INTERFACE_FEATURE:
	AJMP	EMB_NO_SUPPORT_REQ
;----
EMB_CLEAR_DEVICE_FEATURE:
	CLR	EMB_REMOTE_WAKE
;-----
EMB_CLEAR_FEATURE_RET:
EMB_CLR_EPT0_OUT:
	ACALL	SEND_EMB_ZERO_PACKET
	RET

;-------------				;CLEAR ENDPOINR FEATURE
EMB_CLEAR_EPT_FEATURE:
	MOV	A,R5			;I2C_BUFFER+OFFSET_wIndex
	JZ	EMB_CLR_EPT0_OUT	;SHOULD BE UN_STALL
;---
	XRL	A,#080H
	JZ	EMB_CLR_EPT0_IN 	;HUB EPT0 IN
;---
	MOV	A,R5
	XRL	A,#081H
	JZ	EMB_CLR_EPT1_IN 	    ;HUB EPT0 OUT
	AJMP	EMB_NO_SUPPORT_REQ
;----------------
EMB_CLR_EPT0_IN:
	CLR	EMB_EPT0_IN_STALL		 ;HUB EPT0 IN STALL FLAG
	AJMP	EMB_CLEAR_FEATURE_RET
;---------------
EMB_CLR_EPT1_IN:
	CLR	EMB_EPT1_IN_STALL	    ;HUB EPT1 IN STALL FLAG
	AJMP	EMB_CLEAR_FEATURE_RET
;=========================================================
EMB_SET_FEATURE:			    ;3
	MOV	A,R2			    ;I2C_BUFFER+OFFSET_bmReqTyp
	JZ	EMB_SET_DEVICE_FEATURE
	DEC	A
	JZ	EMB_SET_INTERFACE_FEATURE
	DEC	A
	JZ	EMB_SET_EPT_FEATURE
;-----
EMB_SET_INTERFACE_FEATURE:
EMB_NO_SUPPORT_REQ1:
	AJMP	EMB_NO_SUPPORT_REQ
;----
EMB_SET_DEVICE_FEATURE:
	SETB	EMB_REMOTE_WAKE
;-----
EMB_SET_FEATURE_RET:
EMB_SET_EPT0_OUT:
	ACALL	SEND_EMB_ZERO_PACKET
	RET

;---------------			;CLEAR ENDPOINR FEATURE
EMB_SET_EPT_FEATURE:
	MOV	A,R5			;I2C_BUFFER+OFFSET_wIndex
	JZ	EMB_SET_EPT0_OUT	    ;SHOULD BE UN_STALL
;---
	XRL	A,#080H
	JZ	EMB_SET_EPT0_IN 	    ;HUB EPT0 IN
;---
	MOV	A,R5
	XRL	A,#081H
	JZ	EMB_SET_EPT1_IN 	    ;HUB EPT0 OUT
	AJMP	EMB_NO_SUPPORT_REQ
;----------------
EMB_SET_EPT0_IN:
	SETB	EMB_EPT0_IN_STALL	    ;HUB EPT0 IN STALL FLAG
	AJMP	EMB_SET_FEATURE_RET
;---------------
EMB_SET_EPT1_IN:
	SETB	EMB_EPT1_IN_STALL	    ;HUB EPT1 IN STALL FLAG
	AJMP	EMB_SET_FEATURE_RET
;-------

;=======================================================
EMB_GET_DESCRIPTOR:			    ;6
	MOV	A,R4			;I2C_BUFFER+OFFSET_wValue
	JNZ	EMB_NO_SUPPORT_REQ1
	MOV	A,I2C_BUFFER+OFFSET_wValue+1	;I2C_BUFFER+OFFSET_wValue+1
;----
EMB_CHK_DEV_DESC:
	CJNE	A,#01,EMB_CHK_CONF_DESC
	MOV	DPTR,#EMB_STAND_DEV_DESC
	MOV	R0,#I2C_BUFFER+OFFSET_wLenght
;---
	INC	R0			;IF HOST ISSUE OVER 256 BYTES
	MOV	A,@R0
	JNZ	EMB_CHK_DEV_DESC1
	DEC	R0
;---
	MOV	A,@R0
	CLR	C
	SUBB	A,#18
	JC	EMB_SEND_DESCRIPTOR
EMB_CHK_DEV_DESC1:
	MOV	@R0,#18
	AJMP	EMB_SEND_DESCRIPTOR
;----
EMB_CHK_CONF_DESC:
	CJNE	A,#02,EMB_CHK_HID_DESC
	MOV	DPTR,#EMB_CONFIG_DESC
	MOV	R0,#I2C_BUFFER+OFFSET_wLenght
;---
	INC	R0			;IF OVER 256 BYTES
	MOV	A,@R0
	JNZ	EMB_CHK_CONF_DESC1
	DEC	R0
;---
	MOV	A,@R0
	CLR	C
;;;	SUBB	A,#25
	SUBB	A,#022H
	JC	EMB_SEND_DESCRIPTOR
EMB_CHK_CONF_DESC1:
	MOV	@R0,#022H
;----
EMB_SEND_DESCRIPTOR:
	AJMP	EMB_SEND_TO_HOST
;-------------
EMB_CHK_HID_DESC:
	CJNE	A,#021H,EMB_CHK_REPORT_DESC
	MOV	DPTR,#EMB_HID_DESC
	MOV	R0,#I2C_BUFFER+OFFSET_wLenght
;---
	INC	R0			;IF OVER 256 BYTES
	MOV	A,@R0
	JNZ	EMB_CHK_HID_DESC1
	DEC	R0
;---
	MOV	A,@R0
	CLR	C
;;;	SUBB	A,#25
	SUBB	A,#9
	JC	EMB_SEND_DESCRIPTOR
EMB_CHK_HID_DESC1:
	MOV	@R0,#9
	AJMP	EMB_SEND_TO_HOST
;-------------
EMB_CHK_REPORT_DESC:
	CJNE	A,#022H,EMB_NO_SUPPORT_REQ1
	MOV	DPTR,#EMB_REPORT_DESC
	MOV	R0,#I2C_BUFFER+OFFSET_wLenght
;---
	INC	R0			;IF OVER 256 BYTES
	MOV	A,@R0
	JNZ	EMB_CHK_REPORT_DESC1
	DEC	R0
;---
	MOV	A,@R0
	CLR	C
;;;	SUBB	A,#25
	SUBB	A,#REPORT_LENGTH
	JC	EMB_SEND_DESCRIPTOR
EMB_CHK_REPORT_DESC1:
	MOV	@R0,#REPORT_LENGTH
	AJMP	EMB_SEND_TO_HOST
;*******************************************************
;	MOV	R2,I2C_BUFFER+OFFSET_bmReqTyp	;1
;	MOV	R3,I2C_BUFFER+OFFSET_bmReq	;1
;	MOV	R4,I2C_BUFFER+OFFSET_wValue	;2
;	MOV	R5,I2C_BUFFER+OFFSET_wIndex	;2
;	MOV	R6,I2C_BUFFER+OFFSET_wLenght	;2
;*******************************************************
EMB_CLASS_REQUEST:
	MOV	A,R3			;I2C_BUFFER+OFFSET_bmReq
	XRL	A,#9
	JZ	SET_REPORT		;HOST SEND "SET_REPORT" COMMAND
;---
	MOV	A,R3			;THIS COMMAND ONLY FOR CATC DRIVER
	XRL	A,#01
	JZ	GET_REPORT		;GET VCP FEATURE FROM MONITOR
;;	MOV	A,R3
;;	XRL	A,#0AH
;;	JNZ	EMB_C_NO_SUPPORT_REQ
;;	AJMP	EMB_SET_EPT0_OUT	;SEND ZERO PACKET
;----
EMB_C_NO_SUPPORT_REQ:
	AJMP	EMB_NO_SUPPORT_REQ

;=========================================
SET_REPORT:
;;	MOV	A,R4			;
	MOV	A,I2C_BUFFER+OFFSET_wValue+1   ;R4+1
	XRL	A,#03			;IS IT FEATURE REPORT?
	JNZ	EMB_C_NO_SUPPORT_REQ
	MOV	EMB_LAST_COM,#REQ_SET_FEATURE
	MOV	DAC_CHANNEL,R4
	RET
;=========================================
GET_REPORT:
	LJMP	GET_REPORT1



;***********************************************
; SUBROUTINE
;***********************************************


;==============================
EMB_SEND_TO_HOST:
	MOV	A,@R0				;SEND DATA LENGTH
	CLR	C				;R0 = BYTE_TO_SEND
	SUBB	A,#08H
	JNC	EMB_SEND01
	MOV	A,@R0
	MOV	R1,A				;NUMBER BYTES TO SEND
	ADD	A,#2
	MOV	RW_COUNTER,A			;TOTAL BYTES TO SEND
	MOV	EMB_BYTE_TO_SEND,#0		;REMAIN BYTES TO SEND
	AJMP	EMB_SEND02
;-----
EMB_SEND01:
	MOV	EMB_BYTE_TO_SEND,A		    ;REMAIN BYTES
	MOV	R1,#8
	MOV	RW_COUNTER,#0AH 		;TOTAL BYTES TO SEND
;-----
EMB_SEND02:
;----
	MOV	I2C_BUFFER,#WRITE_BUFFER	    ;COMMAND
	MOV	I2C_BUFFER+1,#0 		    ;FIRST BYTE SHOULD BE ZERO
	MOV	I2C_BUFFER+2,R1 		    ;DATA LENGTH
;---
	MOV	R0,#I2C_BUFFER+3
;---
EMB_SEND03:
	CLR	A
	MOVC	A,@A+DPTR
	MOV	@R0,A
	INC	R0
	INC	DPTR
	DJNZ	R1,EMB_SEND03
;---
	MOV	EMB_MSG_DPH,DPH
	MOV	EMB_MSG_DPL,DPL
;---
	ACALL	WRITE_N_BYTE
;---
	RET



;========================================
SEND_ZERO_PACKET:
	MOV	BYTE_TO_SEND,#0
	MOV	I2C_BUFFER,#SEL_EPOINT_0_IN
;-----
SEND_ZERO_P1:
	ACALL	WRITE_COMMAND
;----
	MOV	I2C_BUFFER,#WRITE_BUFFER
	MOV	I2C_BUFFER+1,#0
	MOV	I2C_BUFFER+2,#0
	MOV	I2C_BUFFER+3,#0
	ACALL	WRITE_THREE_BYTE
	RET
;========================================
SEND_EMB_ZERO_PACKET:
	MOV	EMB_BYTE_TO_SEND,#0
	MOV	I2C_BUFFER,#SEL_EPT_EMBF_IN
	AJMP	SEND_ZERO_P1

;========================================
SEND_EP0O_STALL:
	MOV	I2C_BUFFER,#SET_EPT0_OUT_ST	    ;SET ENDPOINT 0
SEND_STALL:
	MOV	I2C_BUFFER+1,#01H		    ;SET ENDPOINT STALL
	ACALL	WRITE_ONE_BYTE
	RET
;========================================
SEND_EP0I_STALL:
	MOV	I2C_BUFFER,#SET_EPT0_IN_ST	   ;SET ENDPOINT 0
	AJMP	SEND_STALL
;========================================
SEND_EMB0I_STALL:
	MOV	I2C_BUFFER,#SET_EMBF_IN_ST	    ;SET EMB EPT0 STALL
	AJMP	SEND_STALL
;========================================
SEND_EMB0O_STALL:
	MOV	I2C_BUFFER,#SET_EMBF_OUT_ST	    ;
	AJMP	SEND_STALL
;========================================

WRITE_NINE_BYTE:
	MOV	RW_COUNTER,#9
	LJMP	WRITE_TO_H11
;----------------------------------------
WRITE_SIX_BYTE:
	MOV	RW_COUNTER,#6
WRITE_N_BYTE:
	LJMP	WRITE_TO_H11
;----------------------------------------
WRITE_THREE_BYTE:
	MOV	RW_COUNTER,#3
	LJMP	WRITE_TO_H11
;----------------------------------------
WRITE_FOUR_BYTE:
	MOV	RW_COUNTER,#4
	LJMP	WRITE_TO_H11
;----------------------------------------
WRITE_TWO_BYTE:
	MOV	RW_COUNTER,#2
	LJMP	WRITE_TO_H11
;----------------------------------------
READ_ONE_BYTE:
	MOV	RW_COUNTER,#1
	LJMP	READ_FROM_H11
;----------------------------------------
READ_TWO_BYTE:
	MOV	RW_COUNTER,#2
	LJMP	READ_FROM_H11
;----------------------------------------
READ_TEN_BYTE:
	MOV	RW_COUNTER,#10
	LJMP	READ_FROM_H11
;----------------------------------------
WRITE_ONE_BYTE:
	MOV	RW_COUNTER,#1
	LJMP	WRITE_TO_H11
;----------------------------------------
WRITE_COMMAND:
	MOV	SLV_ADR,#036H		;WRITE COMMAND
	MOV	CNT_BYTE,#1		;NUMBER OF BYTES TO BE SENT
	LCALL	WRITE_H11_POINT 	;READ 1 BYTES
	RET
;-----
;--------------------------------
; ENDPOINT 0 SEND DATA TO HOST
;--------------------------------
SEND_TO_HOST:
	MOV	A,@R0				;SEND DATA LENGTH
	CLR	C				;R0 = BYTE_TO_SEND
	SUBB	A,#08H
	JNC	SEND01
	MOV	A,@R0
	MOV	R1,A				;NUMBER BYTES TO SEND
	ADD	A,#2
	MOV	RW_COUNTER,A			;TOTAL BYTES TO SEND
	MOV	BYTE_TO_SEND,#0 		;REMAIN BYTES TO SEND
	AJMP	SEND02
;-----
SEND01:
	MOV	BYTE_TO_SEND,A			;REMAIN BYTES
	MOV	R1,#8
	MOV	RW_COUNTER,#0AH 		;TOTAL BYTES TO SEND
;-----
SEND02:
;----
	MOV	I2C_BUFFER,#WRITE_BUFFER	    ;COMMAND
	MOV	I2C_BUFFER+1,#0 		    ;FIRST BYTE SHOULD BE ZERO
	MOV	I2C_BUFFER+2,R1 		    ;DATA LENGTH
;---
	MOV	R0,#I2C_BUFFER+3
;---
SEND03:
	CLR	A
	MOVC	A,@A+DPTR
	MOV	@R0,A
	INC	R0
	INC	DPTR
	DJNZ	R1,SEND03
;---
	MOV	MSG_DPH,DPH
	MOV	MSG_DPL,DPL
;---
	LCALL	WRITE_N_BYTE
;---
	RET
;------------------
EMB_C_NO_SUPPORT_REQ2:
	LJMP	EMB_C_NO_SUPPORT_REQ
;----------------------
	ORG	0810H
;----------------------------------------
GET_REPORT1:
	MOV	A,I2C_BUFFER+OFFSET_wValue+1   ;R4+1
	XRL	A,#03			;IS IT FEATURE REPORT?
	JNZ	EMB_C_NO_SUPPORT_REQ2
	MOV	A,R4
	CJNE	A,#4,CHK_SECU
	MOV	COUNTER,#0
	LJMP	OUT_DATA
;-----
CHK_SECU:
	CJNE	A,#0FDH,CHK_OSD
	AJMP	OUT_PHILIPS

;----
CHK_OSD:
	CJNE	A,#0CAH,CHK_AUTOSIZE
	MOV	COUNTER,#2
	AJMP	OUT_DATA
;----
CHK_AUTOSIZE:
	CJNE	A,#0A2H,CHK_HORIP
	MOV	COUNTER,#1
	AJMP	OUT_DATA
;----
CHK_HORIP:
	CJNE	A,#0A4H,CHK_VERTIP
	MOV	COUNTER,#2
	AJMP	OUT_DATA
;----
CHK_VERTIP:
	CJNE	A,#0A6H,CHK_SYNCT
	MOV	COUNTER,#2
	AJMP	OUT_DATA
;----
CHK_SYNCT:
	CJNE	A,#0A8H,CHK_SCREENP
	MOV	COUNTER,#2
	AJMP	OUT_DATA
;----
CHK_SCREENP:
	CJNE	A,#0AAH,CHK_HORIF
	MOV	COUNTER,#2
	AJMP	OUT_DATA
;----
CHK_HORIF:
	CJNE	A,#0ACH,CHK_VERTIF
	MOV	I2C_BUFFER+5,#07BH
	MOV	COUNTER,#0CH
	AJMP	OUT_DATA1
;----
CHK_VERTIF:
	CJNE	A,#0AEH,CHK_BRIT
	MOV	I2C_BUFFER+5,#17H
	MOV	COUNTER,#070H
	AJMP	OUT_DATA1
;----
CHK_BRIT:
	CJNE	A,#10H,CHK_CONT
	MOV	COUNTER,BRIT_DAC
	AJMP	OUT_DATA

CHK_CONT:
	CJNE	A,#12H,CHK_RR
	MOV	COUNTER,CONT_DAC
	AJMP	OUT_DATA

CHK_RR:
	CJNE	A,#16H,CHK_GG
	MOV	COUNTER,RG_DAC
	AJMP	OUT_DATA

CHK_GG:
	CJNE	A,#18H,CHK_BB
	MOV	COUNTER,GG_DAC
	AJMP	OUT_DATA

CHK_BB:
	CJNE	A,#1AH,CHK_HP
	MOV	COUNTER,BG_DAC
	AJMP	OUT_DATA

CHK_HP:
	CJNE	A,#20H,CHK_HS
	MOV	COUNTER,HP_DAC
	AJMP	OUT_DATA

CHK_HS:
	CJNE	A,#22H,CHK_HPIN
	MOV	COUNTER,HS_DAC
	AJMP	OUT_DATA

CHK_HPIN:
	CJNE	A,#24H,CHK_HPIN_BAL
	MOV	COUNTER,HPIN_DAC
	AJMP	OUT_DATA

CHK_HPIN_BAL:
	CJNE	A,#26H,CHK_VP
	MOV	COUNTER,HBAL_DAC
	AJMP	OUT_DATA

CHK_VP:
	CJNE	A,#30H,CHK_VS
	MOV	COUNTER,VS_DAC
	AJMP	OUT_DATA

CHK_VS:
	CJNE	A,#32H,CHK_TZ
	MOV	COUNTER,VS_DAC
	AJMP	OUT_DATA

CHK_TZ:
	CJNE	A,#42H,CHK_ROT
	MOV	COUNTER,TZ_DAC
	AJMP	OUT_DATA

CHK_ROT:
	CJNE	A,#44H,CHK_DAC
	MOV	COUNTER,ROT_DAC
	AJMP	OUT_DATA

CHK_DAC:
	INC	COUNTER
;----
OUT_DATA:
	MOV	I2C_BUFFER+5,#000H
OUT_DATA1:
	MOV	I2C_BUFFER,#WRITE_BUFFER
	MOV	I2C_BUFFER+1,#0
	MOV	I2C_BUFFER+2,#3
	MOV	I2C_BUFFER+3,R4
	MOV	I2C_BUFFER+4,COUNTER
	MOV	RW_COUNTER,#05H
	LCALL	WRITE_N_BYTE
	RET

GET_VERSION:
	MOV	COUNTER,#0
	AJMP	OUT_DATA
;-----------
;----
OUT_PHILIPS:
	MOV	I2C_BUFFER,#WRITE_BUFFER
	MOV	I2C_BUFFER+1,#0
	MOV	I2C_BUFFER+2,#8
	MOV	I2C_BUFFER+3,#0FDH
	MOV	I2C_BUFFER+4,#50H	    ;'P'
	MOV	I2C_BUFFER+5,#68H	    ;'h'
	MOV	I2C_BUFFER+6,#69H	    ;'i'
	MOV	I2C_BUFFER+7,#6CH	    ;'l'
	MOV	I2C_BUFFER+8,#69H	    ;'i'
	MOV	I2C_BUFFER+9,#70H	    ;'p'
	MOV	I2C_BUFFER+10,#73H	    ;'s'
	MOV	RW_COUNTER,#10
	LCALL	WRITE_N_BYTE
	RET
;==============================
; For Interrupt Pipe
;==============================
KEY_RELEASE:
	CLR	HOT_KEY
KEY_RELEASE1:
	MOV	R1,#0
	AJMP	INT_SEND_TO_HOST

;======================
EMB_ED1_INPUT:
;----
	MOV	I2C_BUFFER,#READ_LAST_EMBF_INT_ST
	LCALL	READ_ONE_BYTE
;------
	MOV	I2C_BUFFER,#SEL_EPT_EMBF_INT	;SELECT EMB EPT0 IN
	LCALL	READ_ONE_BYTE
	MOV	A,I2C_BUFFER
	ANL	A,#00000001B			;IS BUFFER EMPTY?
	JNZ	INT_RET
;-----
	JNB	HOT_KEY,INT_RET 		;NEVER PRESS KEY
	MOV	A,KEY_STATUS
	JZ	KEY_RELEASE			;KEY HAS RELEASED
;-----
	RR	A
	MOV	R1,A
;-----
INT_SEND_TO_HOST:
	MOV	RW_COUNTER,#04			;TOTAL BYTES TO SEND
	MOV	I2C_BUFFER,#WRITE_BUFFER	;COMMAND
	MOV	I2C_BUFFER+1,#0 		;FIRST BYTE SHOULD BE ZERO
	MOV	I2C_BUFFER+2,#2 		;DATA LENGTH
	MOV	I2C_BUFFER+3,#02		;REPORT ID
	MOV	I2C_BUFFER+4,R1
;--------------
INT_WRITE_N_BYTE:				;INTERRUPT PIPE TO HOST
	MOV	I2C_BUFFER,#SEL_EPT_EMBF_INT
	LCALL	WRITE_COMMAND
	MOV	I2C_BUFFER,#WRITE_BUFFER
;-----------
INT_WRITE_TO_H11_1:
	MOV	SLV_ADR,#036H		;WRITE COMMAND
	MOV	CNT_BYTE,#1		;NUMBER OF BYTES TO BE SENT
	LCALL	WRITE_H11_POINT 	;READ 1 BYTES
;----------
	MOV	SLV_ADR,#034H		;WRITE DATA
	MOV	CNT_BYTE,RW_COUNTER
	LCALL	WRITE_H11
;-----
	MOV	I2C_BUFFER,#VALIDATE_BUF    ;READY DATA TO SEND
	LCALL	WRITE_COMMAND
;----
INT_RET:
	RET
;----------------------------------------
;----
INIT_DDC2B:
;-----
	MOV	I2C_BUFFER,#06EH
	MOV	I2C_BUFFER+1,#050H
	MOV	I2C_BUFFER+2,#081H
	MOV	I2C_BUFFER+3,#0F0H
;------
	MOV	A,I2C_BUFFER
	MOV	R0,I2C_BUFFER+1
	XRL	A,R0
	MOV	R0,I2C_BUFFER+2
	XRL	A,R0
	MOV	R0,I2C_BUFFER+3
	XRL	A,R0
	MOV	I2C_BUFFER+4,A
	MOV	I2C_COUNTER,#5
	ACALL	DO_IIC
	RET
;--------------
DEG_FUNC:
	MOV	R2,#00
	AJMP	OUT_DDC2B
;---------------
VESA_COMMAND:
	MOV	A,R3
	XRL	A,#1
	JZ	CURRENT_SETTING
	MOV	A,R3
	XRL	A,#2
	JZ	RESTORE_FACTORY
	MOV	A,R3
	XRL	A,#3
	JZ	RESTORE_SAVED
	RET
;------
RESTORE_FACTORY:
	MOV	R2,#04
	MOV	R3,#00
	AJMP	OUT_DDC2B
;------
RESTORE_SAVED:
	MOV	R2,#0B2H
	MOV	R3,#00
	AJMP	OUT_DDC2B
;------
CURRENT_SETTING:
	MOV	R2,#0B0H
	MOV	R3,#00
	AJMP	OUT_DDC2B



;---------------------------------------
SET_DAC_DATA:
;;	MOV	R3,I2C_BUFFER+2
	MOV	R3,I2C_BUFFER+3
;---
	MOV	R2,DAC_CHANNEL
	MOV	A,R2
	XRL	A,#01H
	JZ	DEG_FUNC
	MOV	A,R2
	XRL	A,#0B0H
	JZ	VESA_COMMAND

;-----------------------------------------
;-----------------------------------------
;-----------------------------------------
OUT_DDC2B:
	MOV	A,R2
	XRL	A,#10H
	JNZ	CK_CONT
	MOV	BRIT_DAC,R3
	AJMP	OUT_DDC2B_1
;-----
CK_CONT:
	MOV	A,R2
	XRL	A,#12H
	JNZ	CK_RG
	MOV	CONT_DAC,R3
	AJMP	OUT_DDC2B_1
;-----
CK_RG:
	MOV	A,R2
	XRL	A,#16H
	JNZ	CK_GG
	MOV	RG_DAC,R3
	AJMP	OUT_DDC2B_1
;-----
CK_GG:
	MOV	A,R2
	XRL	A,#18H
	JNZ	CK_BG
	MOV	GG_DAC,R3
	AJMP	OUT_DDC2B_1
;-----
CK_BG:
	MOV	A,R2
	XRL	A,#1AH
	JNZ	CK_HP
	MOV	BG_DAC,R3
	AJMP	OUT_DDC2B_1
;-----
CK_HP:
	MOV	A,R2
	XRL	A,#20H
	JNZ	CK_HS
	MOV	HP_DAC,R3
	AJMP	OUT_DDC2B_1
;-----
CK_HS:
	MOV	A,R2
	XRL	A,#22H
	JNZ	CK_HPIN
	MOV	HS_DAC,R3
	AJMP	OUT_DDC2B_1
;-----
CK_HPIN:
	MOV	A,R2
	XRL	A,#24H
	JNZ	CK_HPIN_BAL
	MOV	HPIN_DAC,R3
	AJMP	OUT_DDC2B_1
;-----
CK_HPIN_BAL:
	MOV	A,R2
	XRL	A,#26H
	JNZ	CK_VP
	MOV	HBAL_DAC,R3
	AJMP	OUT_DDC2B_1
;-----
CK_VP:
	MOV	A,R2
	XRL	A,#30H
	JNZ	CK_VS
	MOV	VS_DAC,R3
	AJMP	OUT_DDC2B_1
;-----
CK_VS:
	MOV	A,R2
	XRL	A,#32H
	JNZ	CK_TZ
	MOV	VS_DAC,R3
	AJMP	OUT_DDC2B_1
;-----
CK_TZ:
	MOV	A,R2
	XRL	A,#42H
	JNZ	CK_ROT
	MOV	TZ_DAC,R3
	AJMP	OUT_DDC2B_1
;-----
CK_ROT:
	MOV	A,R2
	XRL	A,#44H
	JNZ	OUT_DDC2B_1
	MOV	ROT_DAC,R3
;-----
OUT_DDC2B_1:
	MOV	I2C_BUFFER,#06EH
	MOV	I2C_BUFFER+1,#050H
	MOV	I2C_BUFFER+2,#084H
	MOV	I2C_BUFFER+3,#3
	MOV	I2C_BUFFER+4,R2
	MOV	I2C_BUFFER+5,#00H	;DAC HIGH BYTE
	MOV	I2C_BUFFER+6,R3 	;DAC LOW BYTE
;------
	MOV	A,I2C_BUFFER
	MOV	R0,I2C_BUFFER+1
	XRL	A,R0
	MOV	R0,I2C_BUFFER+2
	XRL	A,R0
	MOV	R0,I2C_BUFFER+3
	XRL	A,R0
	MOV	R0,I2C_BUFFER+4
	XRL	A,R0
	MOV	R0,I2C_BUFFER+5
	XRL	A,R0
	MOV	R0,I2C_BUFFER+6
	XRL	A,R0
	MOV	I2C_BUFFER+7,A
	MOV	I2C_COUNTER,#8
	ACALL	DO_IIC
	RET
;===============================
DO_IIC:
	MOV	R0,#I2C_BUFFER		;SET FIRST RECEIVE DATA BUFFER ADDRESS
	MOV	BYTE_CNT,I2C_COUNTER
;----
	ACALL	HW_GOMASTER		;ACQUIRE BUS AND SEND SLAVE ADDRESS.
	JB	I2C_FAIL,WR_NOK 	;CHECK FOR SLAVE NOT RESPONDING.
;-----
WR_DATA_LOOP:
	MOV	A,@R0			;GET DATA BYTE FROM BUFFER.
	ACALL	HW_SEND_BYTE		;SEND NEXT DATA BYTE.
	INC	R0			;ADVANCE BUFFER POINTER.
	JB	I2C_FAIL,WR_NOK 	;CHECK FOR SLAVE NOT RESPONDING.
	DJNZ	BYTE_CNT,WR_DATA_LOOP	;ALL BYTES SENT?
WR_NOK:
WR_OK:
	ACALL	HW_SEND_STOP		   ;DONE, SEND AN I2C STOP.
	RET
;----------------------
HW_GOMASTER:
	CLR	I2C_FAIL		;CLEAR ERROR STATUS FLAGS.
;---
	JNB	HWSCL,HW_FAULT	       ;CHECK FOR BUS CLEAR.
	JNB	HWSDA,HW_FAULT
	CLR	HWSDA		       ;BEGIN I2C START.
	ACALL	BITDLY
	CLR	HWSCL
;;;	ACALL	BITDLY			;COMPLETE I2C START.
;;;	MOV	A,#06EH 		;GET SLAVE ADDRESS.
;;;	ACALL	HW_SEND_BYTE		;SEND SLAVE ADDRESS.
	RET

;-------------
HW_FAULT:
	SETB	I2C_FAIL		;SET FAULT STATUS
	RET				;AND EXIT.
;-------------
HW_BITDLY:
	RET				;NOPS TO DELAY 5 MICROSECONDS (MINUS 4
;========================================
HW_SEND_BYTE:
	MOV	BITCNT,#8		;SET BIT COUNT.

HW_SBLOOP:
	RLC	A			;SEND ONE DATA BIT.
	MOV	HWSDA,C 	       ;PUT DATA BIT ON PIN.
	ACALL	HW_SCLHIGH		   ;SEND CLOCK.
	ACALL	BITDLY
	CLR	HWSCL
	ACALL	BITDLY
	DJNZ	BITCNT,HW_SBLOOP	   ;REPEAT UNTIL ALL BITS SENT.

	SETB	HWSDA			;RELEASE DATA LINE FOR ACKNOWLEDGE.
	ACALL	HW_SCLHIGH		   ;SEND CLOCK FOR ACKNOWLEDGE.
	ACALL	BITDLY
	JNB	HWSDA,HW_SBEX		  ;CHECK FOR VALID ACKNOWLEDGE BIT.
	SETB	I2C_FAIL		;SET STATUS FOR NO ACKNOWLEDGE.
HW_SBEX:
	CLR	HWSCL		       ;FINISH ACKNOWLEDGE BIT.
	ACALL	BITDLY
	RET
;=======================================
HW_SCLHIGH:
	SETB	HWSCL		       ;Set SCL from our end.
	JNB	HWSCL,$ 	       ;Wait for pin to actually go high.
	RET
;-----------------------------------
HW_SEND_STOP:
	CLR	HWSDA		       ;Get SDA ready for stop.
	ACALL	HW_SCLHIGH		   ;SET CLOCK FOR STOP.
	ACALL	BITDLY
	SETB	HWSDA		       ;SEND I2C STOP.
	ACALL	BITDLY
	RET				;BUS SHOULD NOW BE RELEASED.
;=================================
READ_FROM_H11:
	MOV	SLV_ADR,#036H		;WRITE COMMAND
	MOV	CNT_BYTE,#1		;NUMBER OF BYTES TO BE SENT
	LCALL	WRITE_H11_POINT 	;READ 1 BYTES
;-----
	MOV	SLV_ADR,#035H		;READ DATA
	MOV	CNT_BYTE,RW_COUNTER
	ACALL	READ_H11
	RET

;---------------------------------------
SEL_EMB_BUFFER:
	MOV	I2C_BUFFER,#SEL_EPT_EMBF_IN	   ;SELECT EMBED IN BUFFER
	AJMP	SEL_EPT_COMMAND
;---------------------------------------
; WRITE DATA INTO ENDPOINT 0 INPUT
;---------------------------------------
WRITE_TO_H11:
;-----------
	CLR	VALIDATE_FLAG			    ;IS WRITE BUFFER COMMAND?
	MOV	A,I2C_BUFFER
	XRL	A,#WRITE_BUFFER
	JNZ	WRITE_TO_H11_1
	SETB	VALIDATE_FLAG
;-----------
	JB	SEL_EPT_FLAG,SEL_EMB_BUFFER	    ;OUTPUT TO EMBEDDED BUFFER
;-----------
	MOV	I2C_BUFFER,#SEL_EPOINT_0_IN
SEL_EPT_COMMAND:
	LCALL	WRITE_COMMAND
	MOV	I2C_BUFFER,#WRITE_BUFFER
;-----------
WRITE_TO_H11_1:
	MOV	SLV_ADR,#036H		;WRITE COMMAND
	MOV	CNT_BYTE,#1		;NUMBER OF BYTES TO BE SENT
	ACALL	WRITE_H11_POINT 	;READ 1 BYTES
;----------
	MOV	SLV_ADR,#034H		;WRITE DATA
	MOV	CNT_BYTE,RW_COUNTER
	ACALL	WRITE_H11
;-----
	JNB	VALIDATE_FLAG,WRITE_RET ;ONLY WRITE BUFFER NEED VALIDATE BUFFER
	MOV	I2C_BUFFER,#VALIDATE_BUF    ;READY DATA TO SEND
	LCALL	WRITE_COMMAND
;----
WRITE_RET:
	RET
;----------------------------------------

;***************************************************************
;*   Function = Read_H11
;*	Input = SlvAdr (H11 slave address);
;*		CNT_Byte (Number of Bytes to be sent);
;*
;*     Return = I2C_BUFFER Buffer
;*     Change = R0,R3
;****************************************************************
READ_H11:
	MOV	BYTE_CNT,CNT_BYTE
;----
	ACALL	GOMASTER		;ACQUIRE BUS AND SEND SLAVE ADDRESS.
	JB	I2C_FAIL,READ_NOK	;CHECK FOR SLAVE NOT RESPONDING.
;---
	MOV	R0,#I2C_BUFFER		    ;SET FIRST RECEIVE DATA BUFFER ADDRESS
READ_DATA_LOOP:
	ACALL	RCV_BYTE		;RECEIVE DATA BYTE
	MOV	@R0,A			;PUT RECEIVED DATA TO BUFFER
	INC	R0			;PREPARE NEXT BYTE
	DJNZ	BYTE_CNT,READ_DATA_LOOP ;REPEAT UNTIL ALL BYTES RECEIVED
READ_NOK:
READ_OK:
	ACALL	SEND_STOP		;DONE, SEND AN I2C STOP.
	RET
;========================================
RCV_BYTE:
	MOV	BITCNT,#8		;SET BIT COUNT.
RCV_LOOP:
	ACALL	SCLHIGH 		;READ ONE DATA BIT.
	ACALL	BITDLY
	MOV	C,SDAPIN		;GET DATA BIT FROM PIN.
	RLC	A			;ROTATE BIT INTO RESULT BYTE.
	CLR	SCLPIN
	ACALL	BITDLY
	DJNZ	BITCNT,RCV_LOOP 	;REPEAT UNTIL ALL BITS RECEIVED.
;---
	PUSH	ACC			;SAVE ACCUMULATOR
	MOV	A,BYTE_CNT
	CJNE	A,#1,SRBACK		;CHECK FOR LAST BYTE OF FRAME.
	SETB	SDAPIN			;SEND NO ACKNOWLEDGE ON LAST BYTE.
	SJMP	SRBACLK 		;NO ACK WHEN IT'S LAST BYTE
;----
SRBACK:
	CLR	SDAPIN			;SEND ACKNOWLEDGE BIT.
SRBACLK:
	ACALL	SCLHIGH 		;SEND ACKNOWLEDGE CLOCK.
	POP	ACC			;RESTORE ACCUMULATOR
	ACALL	BITDLY
	CLR	SCLPIN
	SETB	SDAPIN			;CLEAR ACKNOWLEDGE BIT.
	ACALL	BITDLY
	RET

;****************************************************************************
;*   Function = WRITE_H11
;*	Input = SLV_ADR (H11 SLAVE ADDRESS)
;*		SUB_ADR (H11 COMMAND);
;*		CNT_Byte (Number of Bytes to be sent);
;*		@I2C_BUFFER (Output Buffer, Data to be sent, starts from RAM
;*			  location sI2C_BUFFER)
;*     Return = None
;*     Change = R0,R3
;*****************************************************************************
WRITE_H11_POINT:
	MOV	R0,#I2C_BUFFER		    ;SET FIRST RECEIVE DATA BUFFER ADDRESS
WRITE_H11:
	MOV	BYTE_CNT,CNT_BYTE
;----
	ACALL	GOMASTER		;ACQUIRE BUS AND SEND SLAVE ADDRESS.
	JB	I2C_FAIL,WRITE_NOK	;CHECK FOR SLAVE NOT RESPONDING.
;-----
;;;	MOV	R0,#I2C_BUFFER		    ;SET FIRST RECEIVE DATA BUFFER ADDRESS
WRITE_DATA_LOOP:
	MOV	A,@R0			;GET DATA BYTE FROM BUFFER.
	ACALL	SEND_BYTE		;SEND NEXT DATA BYTE.
	INC	R0			;ADVANCE BUFFER POINTER.
	JB	I2C_FAIL,WRITE_NOK	   ;CHECK FOR SLAVE NOT RESPONDING.
	DJNZ	BYTE_CNT,WRITE_DATA_LOOP ;ALL BYTES SENT?
WRITE_NOK:
WRITE_OK:
	ACALL	SEND_STOP		;DONE, SEND AN I2C STOP.
	RET


;========================================
;	SEND SLAVE ADDRESS TO H11
;----------------------------------------
GOMASTER:
	CLR	I2C_FAIL		;CLEAR ERROR STATUS FLAGS.
;---
	JNB	SCLPIN,FAULT		;CHECK FOR BUS CLEAR.
	JNB	SDAPIN,FAULT
	CLR	SDAPIN			;BEGIN I2C START.
	ACALL	BITDLY
	CLR	SCLPIN
	ACALL	BITDLY			;COMPLETE I2C START.
	MOV	A,SLV_ADR		;GET SLAVE ADDRESS.
	ACALL	SEND_BYTE		;SEND SLAVE ADDRESS.
	RET
;-------------
FAULT:
	SETB	I2C_FAIL		;SET FAULT STATUS
	RET				;AND EXIT.
;-------------
BITDLY:
	RET				;NOPS TO DELAY 5 MICROSECONDS (MINUS 4
;========================================
SEND_BYTE:
	MOV	BITCNT,#8		;SET BIT COUNT.

SBLOOP:
	RLC	A			;SEND ONE DATA BIT.
	MOV	SDAPIN,C		;PUT DATA BIT ON PIN.
	ACALL	SCLHIGH 		;SEND CLOCK.
	ACALL	BITDLY
	CLR	SCLPIN
	ACALL	BITDLY
	DJNZ	BITCNT,SBLOOP		;REPEAT UNTIL ALL BITS SENT.

	SETB	SDAPIN			;RELEASE DATA LINE FOR ACKNOWLEDGE.
	ACALL	SCLHIGH 		;SEND CLOCK FOR ACKNOWLEDGE.
	ACALL	BITDLY
	JNB	SDAPIN,SBEX		;CHECK FOR VALID ACKNOWLEDGE BIT.
	SETB	I2C_FAIL		;SET STATUS FOR NO ACKNOWLEDGE.
SBEX:
	CLR	SCLPIN			;FINISH ACKNOWLEDGE BIT.
	ACALL	BITDLY
	RET
;=======================================
SCLHIGH:
	SETB	SCLPIN			;Set SCL from our end.
	JNB	SCLPIN,$		;Wait for pin to actually go high.
	RET
;-----------------------------------
SEND_STOP:
	CLR	SDAPIN			;Get SDA ready for stop.
	ACALL	SCLHIGH 		;SET CLOCK FOR STOP.
	ACALL	BITDLY
	SETB	SDAPIN			;SEND I2C STOP.
	ACALL	BITDLY
	RET				;BUS SHOULD NOW BE RELEASED.
;---
DELAY_20MS:
	MOV	R7,#20
	AJMP	DELAY_1MS
;========================================
DELAY_100MS:
	MOV	R7,#100
;----
DELAY_1MS:
	MOV	A,#165		;(165*3+2+1)*1=498us
DY:
	NOP
	NOP
	NOP
	DEC	A
	JNZ	DY
	DJNZ	R7,DELAY_1MS
DY_RET:
	RET
;------------
HOT_KEY_DT:
	MOV	A,P3
	ANL	A,#00000010B
	XRL	A,#00000010B
	MOV	KEY_STATUS,A		;MOVE KEY INTO BUFFER
	JZ	DY_RET			;NO KEY PRESS,
	JB	HOT_KEY,DY_RET		;KEY PRESSED ALREADY
	SETB	HOT_KEY
	LJMP	KEY_RELEASE1

;------------
	ORG	0E00H
;****************************
;* STANDARD DEVICE DESCRIPTOR
;****************************
STAND_DEV_DESC:
	DB	12h	;0		bLength
	DB	01h	;1		bDescriptorType
	DB	00h	;
	DB	01h	;2-3		version 1.00 USB spec Word
	DB	09h	;4		Class
	DB	00h	;5		bDeviceSubClass
	DB	00h	;6		bDeviceProtocol
	DB	08h	;7		bMaxPacketSize0
	DW	7104h	;8-9		idVendor
	DW	0102h	;a-b		idProduct
	DW	4000h	;c-d		bcdDevice
	DB	00h	;e		iManufacturer
	DB	00h	;f		iProduct
	DB	00h	;10		iSerialNumber
	DB	01h	;11		bNumConfigurations

CONFIG_DESC:
	DB	09h	;0
	DB	02h	;1
	DW	1900h	;2-3
	DB	01h	;4
	DB	01h	;5
	DB	00h	;6
	DB	60h	;7
	DB	05h	;8 10mA

INTERFACE_DESC:
	DB	09h	;0
	DB	04h	;1
	DB	00h	;2
	DB	00h	;3
	DB	01h	;4
	DB	09h	;5
	DB	00h	;6
	DB	00h	;7
	DB	00h	;8

ENDP1_INT_DESC:
	DB	07h	;0
	DB	05h	;1
	DB	81h	;2
	DB	03h	;3
	DW	0100h	;4-5
	DB	0ffh	;6


;****************
;* HUB DECRIPTOR
;****************
HUB_DESC:
	DB	09h	; 0
	DB	29h	; standard HUB descriptortype = 0
	DB	05h	; 5 downstreamports
	DB	04H	; D4D3=00 D2=1 D1D0=0
	DB	00h	; D15--D5=0
	DB	30h	; 48 msec
	DB	0Ah	; 10 mA
	DB	02h	; B7--B4=0 B3-B0=0
	DB	02h	;

;**********************
; H12 HUB DESCRIPTOR
;**********************
HUB_DESC_H12:
	DB	09h	; 0
	DB	29h	; standard HUB descriptortype = 0
	DB	03h	; 3 downstreamports
	DB	04H	; D4D3=00 D2=1 D1D0=0
	DB	00h	; D15--D5=0
	DB	30h	; 48 msec
	DB	0Ah	; 10 mA
	DB	02h	; B7--B4=0 B3-B0=0
	DB	02h	;

;****************************
;* STANDARD DEVICE DESCRIPTOR
;****************************
;;;	ORG	0F00H
EMB_STAND_DEV_DESC:
	DB	12h	;0		bLength
	DB	01h	;1		bDescriptorType
	DB	00h	;
	DB	01h	;2-3		version 1.00 USB spec Word
	DB	00H	;;04h	  ;4		  Class
	DB	00H	;;01h	  ;5		  bDeviceSubClass
	DB	00H	;;0ffh	   ;6		   bDeviceProtocol
	DB	08h	;7		bMaxPacketSize0
	DB   071H,04H  ;8-9		idVendor
	DB   003H,000H	;;03H,01H   ;a-b	    idProduct
	DB    00H,00H	;c-d		bcdDevice
	DB	00h	;e		iManufacturer
	DB	00h	;f		iProduct
	DB	00h	;10		iSerialNumber
	DB	01h	;11		bNumConfigurations

EMB_CONFIG_DESC:
	DB	09h	;0
	DB	02h	;1
	DB     22H,00H ;2-3
	DB	01h	;4
	DB	01h	;5
	DB	00h	;6
	DB	40H	;7
	DB	32	;8

EMB_INTERFACE_DESC:
	DB	09h	;0
	DB	04h	;1
	DB	00h	;2
	DB	00h	;3
	DB	01h	;4
	DB	03h	;5
	DB	00h	;6
	DB	00h	;7
	DB	00h	;8

EMB_HID_DESC:
	DB	09H
	DB	21H
	DB	00H
	DB	01H
	DB	00H
	DB	01H
	DB	22H
	DB	REPORT_LENGTH		;report length
	DB	00H

EMB_ENDP1_INT_DESC:
	DB	07h	;0
	DB	05h	;1
	DB	81h	;2
	DB	03h	;3
	DB	8,0	;4-5
	DB	0FFh	;6

	ORG	0F00H

EMB_REPORT_DESC:
	DB	05H,080H		;Usage Page( Monitor)
	DB	09H,01H 		;Usage ( Monitor Control)
	DB	0A1H,01H		;Collection( Application)
	DB	15h,00H 		;Logical Minimum(0)
	DB	26H,0FFH,00		;Logical Maximum(256)
;----
	DB	75H,08			;Report Size( 8 bit)
	DB	95H,07H 		;Report Counter ( 7)
	DB	85H,0FDH		;Report ID ( FD)
	DB	09H,0FDH		;Usage ( FD)
	DB	0B1H,00H		;FEATURE ( DATA,ARY,ABS)
;----
	DB	05H,082H		;Usage_Page ( VESA Virtual Control)
	DB	75H,10h 		;Report Size( 16 bit)
	DB	95H,01H 		;Report Counter( 1 bytes)
;----
	DB	85H,01H 		;Report ID( 1H)
	DB	09H,01H 		;USGAE ( Degauss)
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
;----
	DB	85H,10H 		;Report ID( 10H)
	DB	09H,10H 		;USGAE ( Brightness)
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
;----
	DB	85H,12H 		;Report ID( 12H)
	DB	09H,12H 		;USGAE ( Contrast)
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
;---
	DB	85H,16H 		;Report ID( 16H)
	DB	09H,16H 		;USGAE ( R GAIN)
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
;----
	DB	85H,18H 		;Report ID( 18H)
	DB	09H,18H 		;USAGE ( G GAIN)
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
;----
	DB	85H,1AH 		;Report ID( 18H)
	DB	09H,1AH 		;USAGE ( B GAIN)
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
;----
	DB	85H,20H 		;Report ID( 20H)
	DB	09H,20H 		;USAGE ( HORIZONTAL POSITION)
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
;----
	DB	85H,22H 		;Report ID( 22H)
	DB	09H,22H 		;USAGE ( HORIZONTAL SIZE)
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
;----
	DB	85H,24H 		;Report ID( 24H)
	DB	09H,24H 		;USGAE ( HORIZONTAL PIN)
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
;----
	DB	85H,30H 		;Report ID( 30H)
	DB	09H,30H 		;USAGE ( VERTICAL POSITION)
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
;----
	DB	85H,32H 		;Report ID( 32H)
	DB	09H,32H 		;USAGE ( VERTICAL SIZE)
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
;----
	DB	85H,42H 		;Report ID( 42H)
	DB	09H,42H 		;USAGE ( TRAPEZOIDAL)
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
;----
	DB	85H,44H 		;Report ID( 44H)
	DB	09H,44H 		;USAGE ( TILT)
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
;----
	DB	85H,0ACH		;Report ID( ACH)
	DB	09H,0ACH		;USAGE ( Hori. Freq)
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
;----
	DB	85H,0AEH		;Report ID( AEH)
	DB	09H,0AEH		;USAGE ( VERTI. FREQ)
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
;-------------
	DB	85H,60H 		;Report ID( 60H)
	DB	09H,60H 		;USAGE ( INPUT SOURCE SELECT)
	DB	0A1H,02 		;COLLECTION ( LOGICAL)
	DB	05H,81H 		;USGAE_PAGE( MONITOR ENUM VALUES)
	DB	09H,01H 		;USAGE ENUM 01
	DB	09H,04H 		;USGAE ENUM 04
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
	DB	0C0H			;END COLLECTION
;-------------
	DB	05H,082H		;Usage_Page ( VESA Virtual Control)
	DB	85H,0A2H		;Report ID( A2H)
	DB	09H,0A2H		;USAGE ( AUTO SIZE CENTER)
	DB	0A1H,02 		;COLLECTION ( LOGICAL)
	DB	05H,81H 		;USGAE_PAGE( MONITOR ENUM VALUES)
	DB	09H,01H 		;USAGE ENUM 01
	DB	09H,02H 		;USGAE ENUM 02
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
	DB	0C0H			;END COLLECTION
;-------------
	DB	05H,082H		;Usage_Page ( VESA Virtual Control)
	DB	85H,0A4H		;Report ID( A4H)
	DB	09H,0A4H		;USAGE ( POLARITY HORI SYNC)
	DB	0A1H,02 		;COLLECTION ( LOGICAL)
	DB	05H,81H 		;USGAE_PAGE( MONITOR ENUM VALUES)
	DB	09H,01H 		;USAGE ENUM 01
	DB	09H,02H 		;USGAE ENUM 02
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
	DB	0C0H			;END COLLECTION
;-------------
	DB	05H,082H		;Usage_Page ( VESA Virtual Control)
	DB	85H,0A6H		;Report ID( A6H)
	DB	09H,0A6H		;USAGE ( POLARITY VERTI SYNC)
	DB	0A1H,02 		;COLLECTION ( LOGICAL)
	DB	05H,81H 		;USGAE_PAGE( MONITOR ENUM VALUES)
	DB	09H,01H 		;USAGE ENUM 01
	DB	09H,02H 		;USGAE ENUM 02
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
	DB	0C0H			;END COLLECTION
;-------------
	DB	05H,082H		;Usage_Page ( VESA Virtual Control)
	DB	85H,0A8H		;Report ID( A8H)
	DB	09H,0A8H		;USAGE ( SYNC TYPE)
	DB	0A1H,02 		;COLLECTION ( LOGICAL)
	DB	05H,81H 		;USGAE_PAGE( MONITOR ENUM VALUES)
	DB	09H,01H 		;USAGE ENUM 01
	DB	09H,02H 		;USGAE ENUM 02
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
	DB	0C0H			;END COLLECTION
;-------------
;;	DB	85H,0AAH		;Report ID( AA)
;;	DB	09H,0AAH		;USAGE ( SCREEN QRIENTATION)
;;	DB	0A1H,02 		;COLLECTION ( LOGICAL)
;;	DB	05H,81H 		;USGAE_PAGE( MONITOR ENUM VALUES)
;;	DB	09H,01H 		;USAGE ENUM 01
;;	DB	09H,02H 		;USGAE ENUM 02
;;	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
;;	DB	0C0H			;END COLLECTION
;-------------
	DB	05H,082H		;Usage_Page ( VESA Virtual Control)
	DB	85H,0CAH		;Report ID( CA)
	DB	09H,0CAH		;USAGE ( On Screen Display)
	DB	0A1H,02 		;COLLECTION ( LOGICAL)
	DB	05H,81H 		;USGAE_PAGE( MONITOR ENUM VALUES)
	DB	09H,01H 		;USAGE ENUM 01	OSD disable
	DB	09H,02H 		;USGAE ENUM 02	OSD enable
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
	DB	0C0H			;END COLLECTION
;-------------
	DB	05H,082H		;Usage_Page ( VESA Virtual Control)
	DB	85H,0B0H		;Report ID( B0H)
	DB	09H,0B0H		;USAGE ( SETTING)
	DB	0A1H,02 		;COLLECTION ( LOGICAL)
	DB	05H,81H 		;USGAE_PAGE( MONITOR ENUM VALUES)
	DB	09H,01H 		;USAGE ENUM 01
	DB	09H,02H 		;USGAE ENUM 02
	DB	09H,03H 		;USGAE ENUM 03
	DB	0B1H,02H		;FEATURE ( DATA,VAR,ABS)
	DB	0C0H			;END COLLECTION

;---------------
	DB	05H,0CH 		;USAGE PAGE( CONSUMER DEVICE)
	DB	25H,07H 		;LOGICAL MAXIMUM (7)
	DB	85H,02H 		;REPORT ID( 2)
	DB	75H,08H 		;REPORT SIZE( 3)
	DB	95H,01H 		;REPORT COUNTER( 1)
	DB	09H,40H 		;USAGE ( MENU)
	DB	09H,0E9H		;USAGE( VOLUME UP)
	DB	09H,0EAH		;USAGE( VOLUME DOWN)
	DB	81H,02H 		;INPUT( DATA,VAR,ABS)
;------------
	DB	0C0H			;END_COLLECTION
;----
	END
